/**
 * Funcin que se encarga de restablecer los valores de los campos de un formulario. Los 
 * campos de tipo 'checkbox' y 'radio' los desmarca, y los campos de tipo 'textarea' los pone 
 * en blanco. Los campo de tipo 'text' reciben un tratamiento especial:si el campo no es de un 
 * identificador, se pone en blanco, pero si el campo es un identificador hay que ponerlo a su 
 * valor inicial por defecto, si lo tiene, seguido del nmero que corresponda a la posicin de 
 * ese campo particular entre todos los de ese mismo tipo. Puede haber en una misma pgina dos 
 * conjuntos de campos identificador, cada uno con un valor inicial por defecto. Recibe el 
 * objeto representando al formulario de la pgina cuyos campos hay que poner a su valor 
 * inicial, el nombre del primer tipo de campo identificador, el valor del identificador por 
 * defecto de ese campo, el nombre del segundo tipo de campo identificador y el valor del 
 * identificador por defecto de ese campo.
 *
 * Puede que no todas las pginas tengan dos conjuntos de campos identificador. Puede que no 
 * tengan ni un primer conjunto de campos identificador, en cuyo caso, los campos 
 * correspondientes llegarn igual a la cadena vaca.
 *
 * @author	David Domnguez
 * @param	formulario		objeto representando al formulario de la pgina cuyos campos se 
 *							quieren poner a sus valores iniciales
 * @param	campo1			nombre comn del primer conjunto de identificadores de la pgina
 * @param	identificador1	nombre base por defecto que toman los identificadores del primer 
 *							grupo de la pgina. Deber estar seguido de un nmero con la 
 *							posicin de cada elemento particular.
 * @param	campo2			nombre comn del segundo conjunto de identificadores de la pgina
 * @param	identificador2	nombre base por defecto que toman los identificadores del segundo 
 *							grupo de la pgina. Deber estar seguido de un nmero con la 
 *							posicin de cada elemento particular.
 */
function resetValues(formulario, campo1, identificador1, campo2, identificador2) {

	// Recorre todos los elementos del formulario
	for (var i = 0; i < formulario.elements.length; i++) {
		// Elemento del formulario a poner a su valor inicial
		var elemento = formulario.elements[i];
		
		// Si es de tipo 'checkbox' lo desmarca
		if (elemento.type == "checkbox") {
			elemento.checked = false;
		}
		// Si es de tipo 'text' hay que iniciar bien los campos que sean de Identificadores
		else if (elemento.type == "text") {
			// Comprueba si es un identificador buscando que su nombre contenga el nombre base 
			// del primer grupo de identificadores a comprobar
			if (elemento.name.lastIndexOf(campo1, elemento.name.length - 1) != -1) {
				// Si el valor por defecto del identificador es vaco, pone el campo vaco
				if (identificador1 == "") {
					elemento.value = "";
				}
				// Si el valor por defecto del identificador no es vaco
				else {
					// Posicin contando desde el final del nombre del campo que contiene un 
					// nmero
					var pos;
					// Caracter del nombre del campo a comprobar si es un nmero
					var numero;
					// Comprueba cuntos caracteres del final del nombre del campo son nmeros
					for (pos = 1, numero = elemento.name.charAt(elemento.name.length - pos); 
							numero <= '9' && numero >= '0';
							numero = elemento.name.charAt(elemento.name.length - pos)){
						pos++;
					}
					// Nmero del final del nombre del campo que indica la posicin de ste 
					// entre todos los campos con el mismo nombre base
					posicion = 
							new Number(elemento.name.substring(elemento.name.length - pos + 1,
							elemento.name.length)) + 1;
					// Nombre por defecto del campo igual al nombre base seguido de la posicin
					elemento.value = identificador1 + posicion;
				}
			} 
			// Comprueba si es un identificador buscando que su nombre contenga el nombre base 
			// del segundo grupo de identificadores a comprobar
			else if (elemento.name.lastIndexOf(campo2, elemento.name.length) != -1){
				// Si el valor por defecto del identificador es vaco, pone el campo vaco
				if (identificador2 == "") {
					elemento.value = "";
				}
				// Si el valor por defecto del identificador no es vaco
				else {
					// Comprueba cuntos caracteres del final del nombre del campo son nmeros
					for (pos = 1, numero = elemento.name.charAt(elemento.name.length - pos); 
							numero <= '9' && numero >= '0'; 
							numero = elemento.name.charAt(elemento.name.length - pos)){
						pos++;
					}
					// Nmero del final del nombre del campo que indica la posicin de ste 
					// entre todos los campos con el mismo nombre base
					posicion = 
							new Number(elemento.name.substring(elemento.name.length - pos + 1,
							elemento.name.length)) + 1;
					// Nombre por defecto del campo igual al nombre base seguido de la posicin
					elemento.value = identificador2 + posicion;
				}
			}
			// Si el campo no es de identificador, se hace su valor igual a vaco
			else {
				elemento.value = "";
			}
		}
		// Si es de tipo 'textarea' la pone igual a vaco
		else if (elemento.type == "textarea") {
			elemento.value = "";
		}
		// si es de tipo 'radio' lo desmarca
		else if (elemento.type == "radio") {
			elemento.checked = false;
		}
	}
}