package control;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import javax.servlet.ServletException;

/**
 * Interfaz de todos los componentes de control implcitos encargados de manejar la lgica 
 * asociada a cada una de las pginas JSP. Su nico mtodo es el que ejecuta toda la lgica. 
 * Todas las clases encargadas de manejar la lgica de las pginas JSP deben implementar esta 
 * clase y su mtodo para que pueda realizarse correctamente.
 * 
 * @author David Domnguez
 *
 */
interface Control {

	/**
	 * Realiza la lgica de control asociada a las JSPs. Deben implementarlo todas las clases 
	 * encargadas realizar esa lgica.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						boolean indicando si se deben seguir ejecutando el resto de 
	 * 								los filtros	o no
	 * @throws	IOException			si hay algn problema con el acceso a ficheros
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see		ControlFilter
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 */
	public boolean doLogic(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException;
}