package control;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Componente de control implcito asociado a la pgina JSP "fin.jsp". Es la lgica asociada a 
 * esa pgina JSP. Su mtodo <code>doLogic</code> se ejecutar siempre que sea llamada la 
 * pgina. Slo se encarga de disponer en la pgina los mensajes de xito o fracaso creando el 
 * tem o el test XML del QTI.
 *  
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Fin implements Control {

	/**
	 * Ser llamado por el filtro "ControlFilter" en el caso de que detecte que la pgina 
	 * requerida es "fin.jsp". Ya que es llamado desde un filtro, ser ejecutado siempre que 
	 * llegue una peticin de un cliente y antes de que la peticin llegue al Servlet (pgina 
	 * JSP compilada a Servlet) que sirve la respuesta al cliente. Se encarga de avisar al 
	 * usuario del xito o fracaso de la creacin del tem XML.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se le enve la respuesta correctamente
	 * @throws	IOException			no debe de darse el caso en esta implementacin de Control
	 * @throws	ServletException	no debe de darse el caso en esta implementacin de Control
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		// Recoge los valores del estado y el mensaje para el usuario, que ser informacin o 
		// aviso de error en funcin del estado
		String mensaje = request.getParameter("Mensaje");
		boolean estado = Boolean.valueOf(request.getParameter("Estado")).booleanValue();
		
		if (estado){
			// Si todo ha ido bien creando el tem
			request.setAttribute("Informacion", mensaje);
			request.setAttribute("Titulo", "xito");
		} else {
			// Si ha habido un error se avisa al usuario
			request.setAttribute("Aviso", mensaje);
			request.setAttribute("Titulo", "Error");
		}
		
		return true;
	}
}