package control;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import xml.test.AssessmentTest;

import utilidades.*;

/**
 * Componente de control implcito asociado a la pgina JSP "fintest.jsp". Es la lgica 
 * asociada a esa pgina JSP. Su mtodo <code>doLogic</code> se ejecutar siempre que sea 
 * llamada la pgina. Permite al usuario elegir el directorio dentro del de test donde 
 * almacenar el test XML y lo intenta crear y almacenar en disco.
 *  
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Fintest implements Control {

	/**
	 * Ser llamado por el filtro <code>ControlFilter</code> en el caso de que detecte que la 
	 * pgina requerida es "fintest.jsp". Ya que es llamado desde un filtro, ser ejecutado 
	 * siempre que llegue una peticin de un cliente y antes de que la peticin llegue al 
	 * Servlet (pgina JSP compilada a Servlet) que sirve la respuesta al cliente. Trata de 
	 * crear el test XML almacenado en sesin. Saca el objeto, que debe ser instancia de <code>
	 * AssessmentTest</code>, de sesin y lo crear en la carpeta que elija el usuario. Si ya 
	 * existe ese nombre de archivo entonces permite modificar el identificador del test e 
	 * introducir un nuevo nombre. Permite al usuario crear nuevos directorios y navegar por 
	 * los directorios. Puede recibir informacin de la pgina de crear nuevos directorios. 
	 * Luego redirecciona la respuesta a "fin.jsp" que informa al usuario del xito o fracaso 
	 * del proceso.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se le enve la respuesta correctamente
	 * @throws	IOException			si hay algn problema accediendo a los directorios o 
	 * 								leyendo su contenido o al escribir el archivo en disco
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	java.io.File
	 * @see 	xml.test.AssessmentTest
	 * @see 	utilidades.MensajeEstado
	 * @see		utilidades.ListaArchivos
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		// Contexto del Servlet
		ServletContext sc =	request.getSession().getServletContext();
		// Directorio base de trabajo (guardado en "web.xml")
		String dirTrabajo = sc.getInitParameter("Directorio Trabajo");
		// Directorio de asignatura si venimos de la pgina de seleccin de asignatura
		String dirAsignatura = request.getParameter("DirectorioAsignatura");
		// Directorio seleccionado a mostrar en este momento
		String dirItemActual = request.getParameter("DirectorioActual");
		// Directorio inicial de los tests (guardado en "web.xml")
		String dirInicial = sc.getInitParameter("Directorio Tests");
		// Directorio superior del actual (null si estamos en el inicial)
		String dirSuperior = null;
		// Aviso recibido de otra pgina a mostrar en esta. Se utiliza tambin para los avisos 
		// de la propia pgina. Se sobreescribe porque los avisos no se pueden dar a la vez.
		String aviso = request.getParameter("Aviso");
		// Informacion recibida de otra pgina a mostrar en esta
		String informacion = request.getParameter("Informacion");
		
		// Si vamos a otra pgina o seguimos en la misma
		boolean salida = false;
		// Si se permite introducir un nuevo identificador
		boolean muestraNuevoIdent = false;
		
		if (dirItemActual == null){
			// Primera vez que se entra en la pgina, as que establece el directorio de inicio
			dirItemActual = "/" + dirAsignatura + dirInicial;
			// Comprueba si existe el directorio base de los tests, y si no existe intenta 
			// crearlo
			File dirItems = new File(sc.getRealPath(dirTrabajo) + "/" + dirAsignatura + 
					dirInicial);
			if (!dirItems.exists())
				if (!dirItems.mkdir())
					throw new IOException("Error: No se pudo crear el directorio base de " +
							"los Tests");
		}

		// Si "Form" es distinto de null se ha pulsado el botn de crear test
		if (request.getParameter("Form") != null){

			HttpSession sesion = request.getSession();
			// Si el objeto sesin no es nuevo la sesin no ha expirado
			if (!sesion.isNew()){
				// Saca de sesin el AssessmentTest creado para crear el archivo XML del test
				AssessmentTest test = (AssessmentTest)sesion.getAttribute("Test2");
				String nombre = test.getIdentificador();	// Nombre del archivo XML a crear
				
				// Estado de la comprobacin del identificador del tem
				boolean correcto = true;
				
			// Si se ha introducido un nuevo identificador crea el archivo con ese nombre, 
				// comprobando antes que sea un nombre correcto
				String identificador = request.getParameter("Identif");
				if (identificador != null && !identificador.equals("")){

					// Caracteres reservados no permitidos
					final String reservados = ";/?:@&=+$,\\*\"<>|";

					// Comprueba que los caracteres del nuevo identificador estn permitidos
			        for(int i = 0; i < identificador.length() && correcto == true; i++) {
			        	char c = identificador.charAt(i);	// Lee el siguiente carcter
			        	int marca = reservados.indexOf(c);
			        	if (marca >= 0 ){
			        		correcto = false;
							aviso = "Error: El carcter '" + reservados.charAt(marca) + 
									"' en la posicin " + (i+1) + " del Identificador no " +
									"est permitido<br />";
							muestraNuevoIdent = true;// Permitir introducir en un nuevo nombre
							// Guarda en request si se muestra el campo de introducir un nuevo 
							// identificador
							request.setAttribute("Nuevo", Boolean.toString(muestraNuevoIdent));

			        	}
			        }
			        if (correcto == true)
			        	nombre = identificador;
				}
				
				// Si el nombre del archivo es correcto se intenta crear el archivo del test
				if (correcto == true){
					File testFile = new File(sc.getRealPath(dirTrabajo) + dirItemActual + "/" + 
							nombre + ".xml");
					if (testFile.exists()){			// Si ya existe un tem con ese nombre
						aviso = "Ya existe un test con ese identificador, debe " +
								"seleccionar otro<br />";
						// Permite introducir en la pgina un nuevo nombre
						muestraNuevoIdent = true;
						// Guarda en request si se muestra el campo de introducir un nuevo 
						// identificador
						request.setAttribute("Nuevo", Boolean.toString(muestraNuevoIdent));
					} else {
						// Si no existe crea el test XML y va a fin.jsp para mostrar el resultado
						test.setIdentificador(nombre);
						test.getTestPart().getSeccion().creaRefItems(testFile);
						MensajeEstado msg = test.creaAssessmentTest(sc.getRealPath(dirTrabajo) + 
								dirItemActual + "/", sc);
						response.sendRedirect("fin.jsp?Estado=" + msg.isEstado() + "&Mensaje=" + 
								msg.getMensaje());
						salida = true;				// Sale de esta pgina
					}
				}
				
			} else{			// Si la sesin ha expirado
				response.sendRedirect("errorfinsesion.jsp");
				salida = true;					// Sale de esta pgina
			}
		}
		
		// Si se sigue en esta pgina muestra los directorios para elegir uno
		if (salida != true){
			// Busca la segunda localizacin del carcter '/' para ver si estamos en el inicio
			int loc = dirItemActual.indexOf('/', 1);
			// Si longitud de la direccin actual es distinta de la que hay del inicio al 
			// segundo  carcter '/' ms la del directorio inicial de los tems no estamos en 
			// el base, y por lo tanto mostramos la opcin de ir al directorio superior
			if (loc + dirInicial.length() != dirItemActual.length())
				// Si no estamos en el inicial guarda el nombre del directorio padre para 
				// poder volver atrs
				dirSuperior = dirItemActual.substring(0, dirItemActual.lastIndexOf('/'));
			
			// Guarda en request los nombres del directorio actual y el del directorio padre
			request.setAttribute("DirectorioActual", dirItemActual);
			request.setAttribute("DirectorioSuperior", dirSuperior);
			
			// Lista los nombres de los directorios del directorio actual y los guarda en 
			// request
			File dirActual = new File(sc.getRealPath(dirTrabajo) + dirItemActual);
			Vector /* String */ dirNames = ListaArchivos.ListaDirectorios(dirActual);
			request.setAttribute("NombresDirectorios",dirNames);
		}

		// Avisos e informacin para el usuario de la pgina
		request.setAttribute("Aviso", aviso);
		request.setAttribute("Informacion", informacion);
		
		return true;
	}
}