package control;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * Componente de control implcito asociado a la pgina JSP "index.jsp". Es la lgica 
 * asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que sea 
 * llamada la pgina. Slo se preocupa de inicializar la sesin del usuario con una nueva para 
 * borrar cualquier objeto antiguo almacenado en sesin. La clase implementa la interfaz <code>
 * Control</code>, implementando su mtodo <code>doLogic</code>, que es el nico que contiene, 
 * y que es el encargado de realizar la lgica asociada a la pgina "index.jsp".
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Index implements Control {

	/**
	 * Reinicia el objeto sesin para eliminar cualquier objeto a desechar almacenado ah. Ser 
	 * llamado por el filtro "ControlFilter" en el caso de que detecte que la pgina requerida 
	 * es "index.jsp". Ya que es llamado desde un filtro, ser ejecutado siempre que llegue una 
	 * peticin de un cliente y antes de que la peticin llegue al Servlet (pgina JSP compilada 
	 * a Servlet) que sirve la respuesta al cliente.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		HttpSession sesion = request.getSession();
		sesion.invalidate();
		sesion = request.getSession(true);

		return true;
	}

}