package control;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import utilidades.EstadoWeb;
import utilidades.ResumenItem;
import utilidades.Identificador;
import utilidades.Comprobaciones;

import xml.test.*;

/**
 * Componente de control implcito asociado a la pgina JSP "introducirpesos.jsp". Es la lgica 
 * asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que sea 
 * llamada la pgina. Su funcin es comprobar que los datos introducidos en la pgina de 
 * introduccin de los pesos de los tems del test sean correctos. La clase implementa la 
 * interfaz <code>Control</code>, implementando su mtodo <code>doLogic</code>, que es el 
 * encargado de realizar la lgica asociada a la pgina "introducirpesos.jsp". Contiene tambin 
 * un mtodo para leer la tabla de pesos de la pgina y otro mtodo para rellenar los pesos que 
 * se dejen vacos.
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Introducirpesos implements Control {

	/**
	 * Realiza la comprobacin de los parmetros que se recogen en la pgina 
	 * "introducirpesos.jsp". Ser llamado por el filtro <code>ControlFilter</code> en el caso 
	 * de que detecte que la pgina requerida es "introducirpesos.jsp". Ya que es llamado desde 
	 * un filtro, ser ejecutado siempre que llegue una peticin de un cliente y antes de que 
	 * la peticin llegue al Servlet (pgina JSP compilada a Servlet) que sirve la respuesta al 
	 * cliente. Comprueba uno por uno todos los parmetros, y, en caso de que sean correctos, 
	 * crea el array de referencias a los tems y guarda el objeto test actualizado en el 
	 * mbito de sesin para utilizarlos ms adelante para terminar de crear el test.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 * @see		utilidades.EstadoWeb
	 * @see		xml.test.AssessmentItemRef
	 * @see		xml.test.AssessmentTest
	 * @see		xml.test.AssessmentSection
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		// Nombre base para los identificadores de los tems por defecto
		final String identificadorBase = "item_";
		request.setAttribute("IdentificadorBase", identificadorBase);
		
		// Objeto sesin para acceder a los objetos ah almacenados
		HttpSession sesion = request.getSession();
		// Si el objeto sesin no es nuevo la sesin no ha expirado
		if (!sesion.isNew()){
			// Array con los tems seleccionados para aadir al test
			ResumenItem[] items = (ResumenItem[])sesion.getAttribute("Items_A_Aadir");
			// Variable para almacenar los errores de la web y dnde colocar el foco
			EstadoWeb estado = new EstadoWeb();
			// Array con los identificadores
			Identificador[] identificadores = new Identificador[items.length];
			// Array de valores de los pesos en caso de tem correcto
			Double[] pesoOk = new Double[items.length];
			// Array de valores de los pesos en caso de tem incorrecto
			Double[] pesoNoOk = new Double[items.length];
			// Se leen los datos de si se mezclan aleatoriamente las respuestas y de si se 
			// seleccionan slo algunos de los tems
			AssessmentTest test = (AssessmentTest)sesion.getAttribute("Test1");
			boolean shuffle = false;
			boolean select = false;
			// Si el objeto Ordering es distinto de null se mezclan aleatoriamente
			if (test.getTestPart().getSeccion().getOrden() != null)
				shuffle = true;
			// Si el objeto Selection es distinto de null hay una seleccin entre los tems
			if (test.getTestPart().getSeccion().getSeleccion() != null)
				select = true;
			// Array de boolean indicando si se dejan en posicin fija los tems
			boolean[] fija = new boolean[items.length];
			// Array de boolean indicando si los tems son requeridos
			boolean[] requerido = new boolean[items.length];
			// Si se sigue en esta pgina o se pasa a la siguiente
			boolean salida = false;
		
			// Si se ha pulsado el botn de crear test
			if(request.getParameter("Crear") != null){
				// Lee los identificadores de los tems comprobando que sean correctos y no
				// haya ninguno repetido, actualizando el estado de la web tras la comprobacin
				Identificador.testIdentificadores(identificadores, "Identif_", estado, request, 
						"");
			
				// Lee los pesos en caso de respuesta correcta comprobndolos
				leePesos(request, "Ok_", pesoOk, estado, "correcto");
				// Los pesos en caso de respuesta correcta deben ser mayores que cero
				for (int i = 0; i < pesoOk.length; i++) {
					if (pesoOk[i] != null && pesoOk[i].doubleValue() <= 0)
						estado.aadeError("El peso correcto del tem " + i + " debe ser un " +
								"nmero mayor que cero<br />", "Ok_" + i);
				}
			
				// lee los pesos en caso de respuesta incorrecta comprobndolos
				leePesos(request, "NoOk_", pesoNoOk, estado, "incorrecto");
				// Los pesos en caso de respuesta incorrecta deben ser iguales o menores que
				// cero
				for (int i = 0; i < pesoNoOk.length; i++) {
					if (pesoNoOk[i] != null && pesoNoOk[i].doubleValue() > 0)
						pesoNoOk[i] = new Double(-pesoNoOk[i].doubleValue());
				}
			
				// Lee los tems en posicin fija si se mezclan aleatoriamente
				if (shuffle == true){
					Comprobaciones.leeCheckBoxes(fija, "Fija_", request);
				}
			
				// Lee los tems requeridos si hay una seleccin
				if (select == true){
					Comprobaciones.leeCheckBoxes(requerido, "Requerido_", request);
					// Comprueba que no se hayan marcado como requeridos ms tems que los que 
					// se deben seleccionar del test
					int numRequeridos = 0;
					for (int i = 0; i < requerido.length; i++)
						if (requerido[i] == true)
							numRequeridos++;
					int seleccionar = 
						test.getTestPart().getSeccion().getSeleccion().getSeleccion();
					if (seleccionar < numRequeridos)
						estado.aadeError("Debe seleccionar menos tems requeridos que el " +
								"nmero de tems a seleccionar para el test, " + seleccionar, 
								"Requerido_0");
				}
			
				// Si todos los pesos introducidos son nmeros decimales vlidos y estn todos 
				// los identificadores y son vlidos
				if (estado.isEstado() == true){
					// Se rellenan los pesos correctos vacos con 1 y incorrectos vacos con 0
					rellenaPesos(pesoOk, 1);
					rellenaPesos(pesoNoOk, 0);
					// Se crea el array de referencias a los tems para el test
					AssessmentItemRef[] itemArray = new AssessmentItemRef[items.length];
					for (int i = 0; i < itemArray.length; i++){
						Weight wOK = new Weight(new Identificador("WeightOK"), 
								pesoOk[i].doubleValue());
						Weight wNOOK = new Weight(new Identificador("WeightNOK"), 
								pesoNoOk[i].doubleValue());
						itemArray[i] = new AssessmentItemRef(identificadores[i], requerido[i], 
								fija[i], items[i].getArchivo(), wOK, wNOOK);
					}
					// Almacena el array de referencias a los tems en la seccin del test
					test.getTestPart().getSeccion().setItemRefArray(itemArray);
					// Almacena en sesin el test para usarlo ms adelante
					sesion.setAttribute("Test2", test);
					// Se redirecciona la respuesta a la siguiente pgina para crear el test
					response.sendRedirect("seleccionasignatura.jsp?Destino=fintest.jsp");
					salida = true;
				}
			}
			// Si no se ha pulsado el botn de crear test es la primera vez que se entra en la 
			// pgina y se inicializan los identificadores de los tems a unos valores por 
			// defecto y se coloca el foco en el primer campo del formulario de peso correcto
			else {
				for (int i = 0; i < items.length; i++)
					if (identificadores[i] == null)
						identificadores[i] = new Identificador(identificadorBase + (i+1));
				
				request.setAttribute("Focus", "Ok_1");
			}

			// Si se sigue en esta pgina se guardan en request todos los parmetros requeridos 
			// y los ya introducidos si se ha introducido alguno
			if (salida == false){
				request.setAttribute("Shuffle", new Boolean(shuffle));
				request.setAttribute("Select", new Boolean(select));
				request.setAttribute("Fija", fija);
				request.setAttribute("Requerido", requerido);
				request.setAttribute("Aviso", estado.getMensaje());
				request.setAttribute("Identif", identificadores);
				request.setAttribute("Ok", pesoOk);
				request.setAttribute("NoOk", pesoNoOk);
				request.setAttribute("Focus", estado.getFocusON());			
				sesion.setAttribute("Items_A_Aadir", items);
			}
		
		} else		// Si la sesin ha expirado
			response.sendRedirect("errorfinsesion.jsp");

		return true;
	}
	
	/**
	 * Lee los pesos introducidos en la pgina de introduccin de pesos del objeto 
	 * <code>HttpServletRequest</code>, con el nombre comn de los campos dado por el <code>
	 * String</code> <code>origen</code> y rellena el array de <code>Double</code> pesos, 
	 * comprobando que sean nmeros decimales correctos, y actualizando el valor del objeto 
	 * <code>estado</code> en caso de error.
	 * 
	 * @param 	request		httpServletRequest de donde leer los pesos introducidos por el 
	 * 						usuario
	 * @param 	origen		string con el nombre base comn de los campos que contienen los 
	 * 						pesos a leer
	 * @param 	pesos		array de Double en el que almacenar los pesos ledos si son nmeros 
	 * 						decimales correctos
	 * @param 	estado		estado de la comprobacin para avisar en caso de error
	 * @param	tipoPeso	string indicando si el peso es tipo correcto o incorrecto para el 
	 * 						aviso al usuario
	 */
	public void leePesos (HttpServletRequest request, String origen, Double[] pesos, 
			EstadoWeb estado, String tipoPeso){
		
		for (int i = 0; i < pesos.length; i++){
			try {
				// Valor del peso en caso de respuesta correcta
				String peso = request.getParameter(origen + i);
				// Si el peso se ha rellenado con un valor
				if (peso != null && !peso.equals(""))
					pesos[i] = new Double(peso);
			}catch (NumberFormatException e){
				estado.aadeError("El peso " + tipoPeso + " del tem " + i + " es incorrecto" +
						"<br />", origen + i);
			}
		}
	}
	
	/**
	 * Rellena las posiciones vacas del array de <code>Double pesos</code> con el 
	 * valor indicado por el parmetro <code>relleno</code>.
	 * 
	 * @param 	pesos		array de objetos Double a terminar de rellenar
	 * @param 	relleno		double con el valor por defecto para rellenar los huecos vacos
	 */
	public void rellenaPesos (Double[] pesos, double relleno){
		for (int i = 0; i < pesos.length; i++)
			if (pesos[i] == null)
				pesos[i] = new Double(relleno);
	}
	
}