package control;

import java.io.File;
import java.io.IOException;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Componente de control implcito asociado a la pgina JSP "nuevodirectorio.jsp". Es la 
 * lgica asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que 
 * sea llamada la pgina. Se encarga de crear un nuevo directorio.
 * 
 * @author 	David Domnguez
 * @see		Control
 * 
 */
public class Nuevodirectorio implements Control {

	/**
	 * Se encarga de crear un nuevo subdirectorio con el nombre indicado por <code>DirNombre
	 * </code> en el directorio <code>DirectorioActual</code>, parmetros ambos que recibe en 
	 * el request. Ser llamado por el filtro <code>ControlFilter</code> en el caso de que 
	 * detecte que la pgina requerida es "nuevodirectorio.jsp". Ya que es llamado desde un 
	 * filtro, ser ejecutado siempre que llegue una peticin de un cliente y antes de que la 
	 * peticin llegue al Servlet (pgina JSP compilada a Servlet) que sirve la respuesta al 
	 * cliente. Comprueba que no exista ningn subdirectorio con ese nombre ya en el directorio 
	 * actual, e intenta crearlo. Avisa al usuario en caso de que ocurra algn fallo. Vuelve a 
	 * la pgina indicada en el parmetro <code>VolverA</code> recibido en el request.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			si hay algn problema accediendo al directorio o al crearlo
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {
		
		// Caracteres reservados no permitidos
		final String reservados = ";/?:@&=+$,\\*\"<>|";

		ServletContext sc =	request.getSession().getServletContext();
		String dirActual = request.getParameter("DirectorioActual");// Directorio actual
		String dirNuevo = request.getParameter("DirNombre");		// Directorio a crear
		String origen = request.getParameter("VolverA");			// Pgina a donde volver
		String aviso = new String("");								// Errores al crearlo
		String informacion = new String("");						// Informacin de xito
		boolean correcto = true;					// Si es correcto el nombre del directorio
		
		// Pone en request el directorio actual y a donde volver por si hay algn fallo que 
		// siga estando disponible o al pulsar cancelar
		request.setAttribute("DirectorioActual", dirActual);
        String atributos;	// Atributos que se pasan en la URL para la pgina siguiente
        if (origen.indexOf('?') == -1)	// Si el origen no tiene '?' se pone uno el primero
        	atributos = "?DirectorioActual=" + dirActual;
        else							// Si ya tiene el '?', sigue aadiendo datos
        	atributos = "&DirectorioActual=" + dirActual;
		
		request.setAttribute("VolverA", origen + atributos);
		// Si no es la primera vez que visita la pgina y se ha introducido el nombre del 
		// directorio
		if (dirNuevo != null && !dirNuevo.equals("")){
			// Comprueba que los caracteres del nombre del directorio estn permitidos
	        for(int i = 0; i < dirNuevo.length() && correcto == true; i++) {
	        	char c = dirNuevo.charAt(i);	// Lee el siguiente carcter
	        	int marca = reservados.indexOf(c);
	        	if (marca >= 0 ){
					aviso = "Error: El carcter '" + reservados.charAt(marca) + "' en la " +
							"posicin " + (i+1) + " no est permitido<br />";
					correcto = false;
	        	}
	        }

	        if (correcto == true){
	        	String dirTrabajo = sc.getInitParameter("Directorio Trabajo");
	        	File dir = new File( sc.getRealPath(dirTrabajo)+ dirActual + "/" + dirNuevo);
	        	if (!dir.exists()) {		// Si no existe el directorio intenta crearlo
	        		if (dir.mkdir()){
	        			informacion = "Directorio creado con xito<br />";
	        		} else{				// Si el directorio no pudo ser creado avisa al usuario
	        			aviso = "El directorio no pudo ser creado<br />";
	        		}
	        	} else {					// Si el directorio ya existe avisa al usuario
	        		aviso = "El directorio ya existe<br />";
	        	}
	        }
	        // Redirecciona la respuesta a la pgina desde la que se llama a esta, pasndole 
	        // el directorio actual, y los avisos o informacin de xito relacionadas con la 
	        // creacin del directorio
	        if (origen.indexOf('?') == -1)	// Si el origen no tiene '?' se pone uno el primero
	        	atributos = "?Aviso=" + aviso + "&Informacion=" + informacion + 
	        			"&DirectorioActual=" + dirActual;
	        else							// Si ya tiene el '?', sigue aadiendo datos
	        	atributos = "&Aviso=" + aviso + "&Informacion=" + informacion + 
	        			"&DirectorioActual=" + dirActual;
	        	
			response.sendRedirect(origen + atributos);
		}
		
		return true;
	}
}