package control;

import utilidades.*;

import xml.items.ChoiceXML;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.*;

/**
 * Componente de control implcito asociado a la pgina JSP "respuestaschoice.jsp". Es la 
 * lgica asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que 
 * sea llamada la pgina. Su funcin es comprobar que todos los parmetros finales de la toma 
 * de datos de una pregunta tipo "Choice" sean correctos, en cuyo caso redirecciona la 
 * respuesta a la pgina "seleccionasignatura.jsp" para elegir donde guardar el tem. La clase 
 * implementa la interfaz <code>Control</code>, implementando su mtodo <code>doLogic</code>, 
 * que es el encargado de realizar la lgica asociada a la pgina "respuestaschoice.jsp". 
 * Tiene otro mtodo que simplemente se ejecuta si los parmetros son correctos para 
 * redireccionar la respuesta y terminar de crear el objeto <code>ChoiceXML</code>.
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Respuestaschoice implements Control {

	/**
	 * Realiza la comprobacin de los parmetros que se recogen en la pgina 
	 * "respuestaschoice.jsp". Ser llamado por el filtro <code>ControlFilter</code> en el 
	 * caso de que detecte que la pgina requerida es "respuestaschoice.jsp". Ya que es 
	 * llamado desde un filtro, ser ejecutado siempre que llegue una peticin de un cliente y 
	 * antes de que la peticin llegue al Servlet (pgina JSP compilada a Servlet) que sirve la 
	 * respuesta al cliente. Comprueba uno por uno todos los parmetros, y, en caso de que sean 
	 * correctos, llama al mtodo <code>terminaChoice</code> para terminar de crear el objeto 
	 * del tem y redireccionar la respuesta.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 * @see		utilidades.EstadoWeb
	 * @see		utilidades.EnteroPositivo
	 * @see		utilidades.Identificador
	 * @see		utilidades.Comprobaciones
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		// Nombre base para los identificadores de las respuestas por defecto
		final String identificadorBase = "Respuesta";
		request.setAttribute("IdentificadorBase", identificadorBase);
		
		// Estos elementos siempre estarn presentes en request al venir de la pgina anterior
		String num_Resp = request.getParameter("Num_Resp");	// Nmero de respuestas del tem
		int numResp = Integer.parseInt(num_Resp);			// Nmero de respuestas en un entero
		String shuff = request.getParameter("Shuff");		// Mezclar respuestas s o no

		// Array con los identificadores
		Identificador[] identificadores = new Identificador[numResp];
		
		//Comprueba si es la primera vez que se entra en la pgina
		if(request.getParameter("Form") != null){
		
			String[] respuestas = new String [numResp];	// Array con las respuestas
			// Estado de la peticin web. Almacena si son correctos los parmetros, los 
			// mensajes de aviso al usuario y dnde colocar el foco en la pgina
			EstadoWeb estado = new EstadoWeb();		
			
			int i;
			// Lee las respuestas introducidas, y si alguna est vaca aade el cdigo de error
			for(i = 0; i < numResp; i++){
				respuestas[i] = request.getParameter("Respuesta"+i);
				if (respuestas[i] == null || respuestas[i].equals(""))
					estado.aadeError("Error: la respuesta " + (i+1) + " no puede quedar " +
							"vaca<br />", "Respuesta" + i);
			}
			
			// Lee los identificadores de las respuestas comprobando que sean correctos y no 
			// haya ninguno repetido, devolviendo el estado de la web tras la comprobacin
			Identificador.testIdentificadores(identificadores, "Identificador", estado,	
					request, "");
			
			//Lee los checkbox de respuesta en posicin fija
			boolean[] respFija = new boolean[numResp];
			Comprobaciones.leeCheckBoxes(respFija, "Fija", request);
		
			// Comprueba los checkbox de respuesta correcta que al menos uno est marcado
			// y suma el nmero total de respuestas correctas
			int numCorrectas = 0;						// Nmero total de respuesas correctas
			boolean[] respCorrecta = new boolean[numResp];	// Respuesta correcta o no
			numCorrectas = Comprobaciones.sumaCorrectas(respCorrecta, "Correcto", request);
			if (numCorrectas == 0)	// Sin ningn checkbox de respuesta correcta est marcado
				estado.aadeError("Debe marcar al menos una respuesta correcta<br />", 
						"Correcto0");
			
			// Lee el valor introducido del nmero mximo de elecciones
			String max = request.getParameter("MaxChoices");
			EnteroPositivo maxChoices = new EnteroPositivo(max);
			// Comprueba que el valor introducido sea un valor correcto, mayor que el nmero 
			// de respuestas correctas o cero, y menor que el nmero de respuestas posibles
			Comprobaciones.testMaximo(maxChoices, estado, "MaxChoices", numCorrectas, numResp);
			
			// Si todo ha ido bien llama al mtodo que finaliza el tem y redirecciona al 
			// cliente
			if (estado.isEstado() == true){
				terminaChoice(request, response, respuestas, identificadores, maxChoices, 
						respCorrecta, respFija);					
			} else {
				// Si ha habido algn fallo pone los datos que ya ha introducido el usuario 
				// junto a los mensajes de aviso y el foco en el primer fallo encontrado
				request.setAttribute("Respuestas",respuestas);
				request.setAttribute("Identificador",identificadores);
				request.setAttribute("MaxChoices",max);
				request.setAttribute("Correcto",respCorrecta);
				request.setAttribute("Fija",respFija);
				request.setAttribute("Aviso",estado.getMensaje());
				request.setAttribute("Focus",estado.getFocusON());
			}
		} else {
			// Si es la primera vez que se visita la pgina pone el foco en el primer campo
			request.setAttribute("Focus", "MaxChoices");
			// Se inicializan los identificadores de las respuestas a unos valores por defecto
			for (int i = 0; i < identificadores.length; i++){
				identificadores[i] = new Identificador(identificadorBase + (i+1));
				request.setAttribute("Identificador",identificadores);
			}
		}
		
		// El nmero de respuestas y si se mezclan las respuestas aleatoriamente hacen falta 
		// siempre, sea la primera vez que visita la pgina o no y pone el foco en el primer 
		// campo del formulario
		request.setAttribute("Num_Resp", num_Resp);
		request.setAttribute("Shuff",shuff);

		return true;
	}
	
	/**
	 * Este mtodo se ejecuta una vez que se comprueba que todos los parmetros son correctos. 
	 * Usa el objeto <code>ChoiceXML</code> creado anteriormente y almacenado en sesin para 
	 * guardar el resto de los datos asociados al tem, redireccionando la respuesta a 
	 * la pgina "seleccionasignatura.jsp" para que el usuario seleccione una asignatura en la 
	 * que guardar el tem XML. Si el objeto <code>ChoiceXML</code> no se ha podido recuperar 
	 * de sesin porque sta ha expirado, redirecciona la respuesta a una pgina para informar 
	 * al usuario.
	 * 
	 * @param 	req				peticin del cliente para acceder a la sesin
	 * @param 	res				respuesta para el cliente para redireccionarlo a otra pgina
	 * @param 	respuestas		array de string con cada una de las respuestas
	 * @param 	identif			array de identificador con los identificadores de las 
	 * 							respuestas
	 * @param 	max				enteroPositivo con el nmero mximo de respuestas elegibles
	 * @param 	correct			array de boolean marcando las respuestas que son correctas
	 * @param 	fija			array de boolean marcando las respuestas que son fijas
	 * @throws 	IOException		si hay algn error al redireccionar la respuesta
	 * @see		xml.items.ChoiceXML
	 * @see		javax.servlet.http.HttpSession
	 * @see		javax.servlet.http.HttpServletResponse#sendRedirect(java.lang.String)
	 */
	private void terminaChoice(HttpServletRequest req, HttpServletResponse res, 
			String[] respuestas, Identificador[] identif, EnteroPositivo max, 
			boolean [] correct,	boolean[] fija) throws IOException{

		HttpSession sesion = req.getSession();
		// Si el objeto sesin no es nuevo la sesin no ha expirado
		if (!sesion.isNew()){	
			// Rellena los campos que faltan en el objeto ChoiceXML que se cre en la 
			// primera parte de la toma de datos
			ChoiceXML chXML = (ChoiceXML)sesion.getAttribute("Choice");
			chXML.setRespuestas(respuestas);
			chXML.setIdentificadores(identif);
			chXML.setMaxChoices(max.getEnteroPositivo());
			chXML.setCorrecto(correct);
			chXML.setFija(fija);
			// Almacena el tem modificado en sesin para utilizarlo ms adelante
			sesion.setAttribute("AItem", chXML);
			// Redirecciona la respuesta a "seleccionasignatura.jsp" indicndole la direccin 
			// a la que debe ir despus
			res.sendRedirect("seleccionasignatura.jsp?Destino=finitem.jsp");
		} else			// Si la sesin ha expirado
			res.sendRedirect("errorfinsesion.jsp");					

	}
}