package control;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import utilidades.*;

import xml.items.GapmatchXML;

/**
 * Componente de control implcito asociado a la pgina JSP "respuestasgapmatch.jsp". Es la 
 * lgica asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que 
 * sea llamada la pgina. Su funcin es comprobar que todos los parmetros finales de la toma 
 * de datos de una pregunta tipo "Gap Match" sean correctos, en cuyo caso redirecciona la 
 * respuesta a la pgina "seleccionasignatura.jsp" para elegir donde guardar el tem. La clase 
 * implementa la interfaz <code>Control</code>, implementando su mtodo <code>doLogic</code>, 
 * que es el encargado de realizar la lgica asociada a la pgina "respuestaschoice.jsp". 
 * Tiene otro mtodo que simplemente se ejecuta si los parmetros son correctos para 
 * redireccionar la respuesta y terminar de crear el objeto <code>GapmatchXML</code>.
 * 
 * @author 	David Domnguez
 * @see		Control
 * 
 */
public class Respuestasgapmatch implements Control {

	/**
	 * Realiza la comprobacin de los parmetros que se recogen en la pgina 
	 * "respuestasgapmatch.jsp". Ser llamado por el filtro <code>ControlFilter</code> en el 
	 * caso de que detecte que la pgina requerida es "respuestasgapmatch.jsp". Ya que es 
	 * llamado desde un filtro, ser ejecutado siempre que llegue una peticin de un cliente y 
	 * antes de que la peticin llegue al Servlet (pgina JSP compilada a Servlet) que sirve la 
	 * respuesta al cliente. Comprueba uno por uno todos los parmetros, y, en caso de que sean 
	 * correctos, llama al mtodo <code>terminaGapmatch</code> para terminar de crear el objeto 
	 * del tem y redireccionar la respuesta.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 * @see		utilidades.EstadoWeb
	 * @see		utilidades.EnteroPositivo
	 * @see		utilidades.Identificador
	 * @see		utilidades.Comprobaciones
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		// Nombre base para los identificadores de los huecos por defecto
		final String identificadorBaseHueco = "Hueco";
		request.setAttribute("IdentificadorBaseHueco", identificadorBaseHueco);
		// Nombre base para los identificadores de las opciones por defecto
		final String identificadorBaseOpcion = "Opcion";
		request.setAttribute("IdentificadorBaseOpcion", identificadorBaseOpcion);
		
		// Estos elementos siempre estarn presentes en request al venir de la pgina anterior
		String numOpciones = request.getParameter("NumOpc");	// Nmero de opciones del tem
		String numHuecos = request.getParameter("NumHuec");		// Nmero de huecos del tem
		String shuff = request.getParameter("Shuffle");			// Mezclar respuestas s o no
		int numOpc = Integer.parseInt(numOpciones);		// Nmero de opciones y 
		int numHuec = Integer.parseInt(numHuecos);		// de huecos en un entero
		// Identificadores de las opciones
		Identificador[] identOpciones = new Identificador[numOpc];
		// Identificadores de los huecos
		Identificador[] identHuecos = new Identificador[numHuec];

		//Comprueba si es la primera vez que se entra en la pgina
		if(request.getParameter("Form") != null){
			String[] opciones = new String [numOpc];		// Opciones disponibles
				
			// Estado de la peticin web. Almacena si son correctos los parmetros, los 
			// mensajes de aviso al usuario y dnde colocar el foco en la pgina
			EstadoWeb estado = new EstadoWeb();		

			int i;
			// Lee los fragmentos de texto que acompaan a los huecos
			String[] fragmentosTexto = new String[numHuec + 1];
			for (i = 0; i < (numHuec + 1); i++)
				fragmentosTexto[i] = request.getParameter("Texto" + i);
			
			// Lee las opciones introducidas, y si alguna est vaca aade el cdigo de error
			for(i = 0; i < numOpc; i++){
				opciones[i] = request.getParameter("Opcion"+i);
				if (opciones[i] == null || opciones[i].equals(""))
					estado.aadeError("Error: la opcin " + (i+1) + " est vaca<br />", 
							"Opcion"+i);
			}
			
			// Lee los identificadores de las opciones comprobando que sean correctos y no 
			// haya ninguno repetido, devolviendo el estado de la web tras la comprobacin
			Identificador.testIdentificadores(identOpciones, "IdentOpc", estado, request, 
					"En las opciones ");
			
			// Lee los identificadores de los huecos comprobando que sean correctos y no haya 
			// ninguno repetido, devolviendo el estado de la web tras la comprobacin
			Identificador.testIdentificadores(identHuecos, "IdentHuec", estado, request, 
					"En los huecos ");
			
			//Lee los checkbox de opciones en posicin fija
			boolean[] respFijaOpc = new boolean[numOpc];
			Comprobaciones.leeCheckBoxes(respFijaOpc, "Fija", request);
					
			// Comprueba los botones de respuesta correcta que uno est marcado en cada fila
			String[] respCorrecta = new String[numHuec];// Contendr el nmero de la columna 
														// de la opcin que es correcta
			// Tambin suma el nmero de elementos correctos en cada columna 
			int[] numCorrectasOpc = new int[numOpc];	// Nmero de correctos en cada opcin
			for (int j = 0; j < numOpc; j++)
				numCorrectasOpc[j] = 0;		// Inicializa el nmero de correctas en la columna
			for (i = 0; i < numHuec; i++){
				respCorrecta[i] = request.getParameter("Correcto"+i);
				if (respCorrecta[i] == null)
					// Si no se ha marcado ninguna correcta en esa fila pone el foco en el 
					// identificador del hueco correspondiente ya que no se puede en el botn 
					// de radio
					estado.aadeError("Debe marcar la opcin correcta para  el hueco " + 
							(i+1) +	"<br />", "IdentHuec" + i);
				else	// Aumenta en uno el nmero de correctas de la columna marcada correcta
					numCorrectasOpc[Integer.parseInt(respCorrecta[i])]++;
			}
			
			// Lee los valores de la tabla de valores mximos de eleccin de las opciones
			EnteroPositivo[] maxChoicesOpc = new EnteroPositivo[numOpc];					
			String[] max = new String[numOpc];
			for (i = 0; i< numOpc; i++)
				max[i] = request.getParameter("Max"+i);
			Comprobaciones.testTablaMaximos(maxChoicesOpc, "Max", estado, max, 
					"de las opciones", "columna", numCorrectasOpc, numHuec);
			
			//Si todo est bien llama al mtodo que finaliza el tem y redirecciona al cliente
			if (estado.isEstado() == true){
				terminaGapmatch(request, response, fragmentosTexto, opciones, identHuecos, 
						identOpciones, maxChoicesOpc, respCorrecta, respFijaOpc);
			} else {
				// Si ha habido algn fallo pone los datos que ya ha introducido el usuario 
				// junto a los mensajes de aviso y el foco en el primer fallo encontrado
				request.setAttribute("Texto", fragmentosTexto);
				request.setAttribute("Opcion", opciones);
				request.setAttribute("IdentHuec", identHuecos);
				request.setAttribute("IdentOpc", identOpciones);
				request.setAttribute("Max", max);
				request.setAttribute("Correcto", respCorrecta);
				request.setAttribute("Fija", respFijaOpc);
				request.setAttribute("Aviso", estado.getMensaje());
				request.setAttribute("Focus",estado.getFocusON());
			}
		} else {
			// Si es la primera vez que se visita la pgina pone el foco en el primer campo
			request.setAttribute("Focus", "Texto0");
			// Se inicializan los identificadores de las opciones y los huecos a unos valores 
			// por defecto
			for (int i = 0; i < identOpciones.length; i++)
				identOpciones[i] = new Identificador(identificadorBaseOpcion + (i+1));
			for (int i = 0; i < identHuecos.length; i++)
				identHuecos[i] = new Identificador(identificadorBaseHueco + (i+1));
			request.setAttribute("IdentHuec", identHuecos);
			request.setAttribute("IdentOpc", identOpciones);
		}
		
		// El nmero de huecos, el nmero de opciones y si se mezclan las respuestas 
		// aleatoriamente hacen falta siempre, sea la primera vez que visita la pgina o no y 
		// pone el foco en el primer campo del formulario
		request.setAttribute("NumOpc",numOpciones);
		request.setAttribute("NumHuec",numHuecos);
		request.setAttribute("Shuffle",shuff);

		return true;
	}
	
	/**
	 * Este mtodo se ejecuta una vez que se comprueba que todos los parmetros son correctos. 
	 * Usa el objeto <code>GapmatchXML</code> creado anteriormente y almacenado en sesin 
	 * para guardar el resto de los datos asociados a la pregunta, redireccionando la 
	 * respuesta a la pgina "seleccionasignatura.jsp" para que el usuario seleccione una 
	 * asignatura en la que guardar el tem XML. Si el objeto <code>GapmatchXML</code> no se 
	 * ha podido recuperar de sesin porque sta ha expirado, redirecciona la respuesta a una 
	 * pgina para informar al usuario.
	 * 
	 * @param 	req				peticin del cliente para acceder a la sesin
	 * @param 	res				respuesta para el cliente para redireccionarlo a otra pgina
	 * @param	textos			array con los textos que rodean a los huecos
	 * @oaram	opc				array de opciones disponibles
	 * @param 	iHuec			array de identificador con los identificadores de los huecos
	 * @param 	iOpc			array de identificador con los identificadores de las opciones
	 * @param 	maxOpc			array de enteroPositivo con el nmero mximo de opciones que 
	 * 							puede seleccionar cada hueco
	 * @param 	correct			array de string con el nmero de la opcin correcta de cada 
	 * 							hueco
	 * @param 	fija			array de boolean marcando las opciones que son fijas
	 * @throws 	IOException		si hay algn error al redireccionar la respuesta
	 * @see		xml.items.GapmatchXML
	 * @see		javax.servlet.http.HttpSession
	 * @see		javax.servlet.http.HttpServletResponse#sendRedirect(java.lang.String)
	 */
	private void terminaGapmatch(HttpServletRequest req, HttpServletResponse res, 
			String[] textos, String[] opc, Identificador[] iHuec, Identificador[] iOpc, 
			EnteroPositivo[] maxOpc, String[] correct, boolean[] fija) throws IOException{
		HttpSession sesion = req.getSession();
		// Si el objeto sesin no es nuevo la sesin no ha expirado
		if (!sesion.isNew()){	
			// Rellena los campos que faltan en el objeto ChoiceXML que se cre en la 
			// primera parte de la toma de datos
			GapmatchXML gmXML = (GapmatchXML)sesion.getAttribute("GapMatch");
			gmXML.setTextos(textos);
			gmXML.setOpciones(opc);
			gmXML.setIdentHuecos(iHuec);
			gmXML.setIdentOpciones(iOpc);
			gmXML.setMaxOpciones(maxOpc);
			gmXML.setRespCorrecta(correct);
			gmXML.setFija(fija);
			// Almacena el tem modificado en sesin para utilizarlo ms adelante
			sesion.setAttribute("AItem", gmXML);
			// Redirecciona la respuesta a "seleccionasignatura.jsp" indicndole la direccin 
			// a la que debe ir despus
			res.sendRedirect("seleccionasignatura.jsp?Destino=finitem.jsp");
		} else			// Si la sesin ha expirado
			res.sendRedirect("errorfinsesion.jsp");

	}
}