package control;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.*;

import xml.items.InlinechoiceXML;

import utilidades.*;

/**
 * Componente de control implcito asociado a la pgina JSP "respuestasinlinechoice.jsp". Es 
 * la lgica asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre 
 * que sea llamada la pgina. Su funcin es comprobar que todos los parmetros finales de la 
 * toma de datos de una pregunta tipo "Inline Choice" sean correctos, en cuyo caso redirecciona 
 * la respuesta a la pgina "seleccionasignatura.jsp" para elegir donde guardar el tem. La 
 * clase implementa la interfaz <code>Control</code>, implementando su mtodo <code>doLogic
 * </code>, que es el encargado de realizar la lgica asociada a la pgina
 * "respuestasinlinechoice.jsp".Tiene otro mtodo que simplemente se ejecuta si los parmetros 
 * son correctos para redireccionar la respuesta y terminar de crear el objeto <code>
 * InlinechoiceXML</code>.
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Respuestasinlinechoice implements Control {

	/**
	 * Realiza la comprobacin de los parmetros que se recogen en la pgina 
	 * "respuestasinlinechoice.jsp". Ser llamado por el filtro <code>ControlFilter</code> en 
	 * el caso de que detecte que la pgina requerida es "respuestasinlinechoice.jsp". Ya que 
	 * es llamado desde un filtro, ser ejecutado siempre que llegue una peticin de un 
	 * cliente y antes de que la peticin llegue al Servlet (pgina JSP compilada a Servlet) 
	 * que sirve la respuesta al cliente. Comprueba uno por uno todos los parmetros, y, en 
	 * caso de que sean correctos, llama al mtodo <code>terminaInlinechoice</code> para 
	 * terminar de crear el objeto del tem y redireccionar la respuesta.
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 * @see		utilidades.EstadoWeb
	 * @see		utilidades.Identificador
	 * @see		utilidades.Comprobaciones
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		// Nombre base para los identificadores de las opciones por defecto
		final String identificadorBase = "Opcion";
		request.setAttribute("IdentificadorBase", identificadorBase);
		
		// Estos elementos siempre estarn presentes en request al venir de la pgina anterior
		String num_Opc = request.getParameter("Num_Opc");		// Nmero de respuestas
		String shuff = request.getParameter("Shuff");			// Mezclar respuestas s o no
		int numOpc = Integer.parseInt(num_Opc);			// Nmero de opciones en entero
		// Array con los identificadores de las opciones de respuesta
		Identificador[] identificadores = new Identificador[numOpc];
			
		//Comprueba si es la primera vez que se entra en la pgina
		if(request.getParameter("Form") != null){

			// Array con las opciones posibles de respuesta
			String[] opciones = new String [numOpc];
		
			// Estado de la peticin web. Almacena si son correctos los parmetros, los 
			// mensajes de aviso al usuario y dnde colocar el foco en la pgina
			EstadoWeb estado = new EstadoWeb();		
			
			// Recoge el valor de los dos campos de texto que pueden estar vacos
			String texto1 = request.getParameter("Texto1");
			String texto2 = request.getParameter("Texto2");
			
			int i;
			// Lee las opciones introducidas, y si alguna est vaca aade el cdigo de error
			for(i = 0; i < numOpc; i++){
				opciones[i] = request.getParameter("Opciones"+i);
				if (opciones[i] == null || opciones[i].equals(""))
					estado.aadeError("Error: la opcin " + (i+1) + " no puede ser vaca" +
							"<br />", "Opciones" + i);
			}
			
			// Lee los identificadores de las opciones comprobando que sean correctos y no 
			// haya ninguno repetido, devolviendo el estado de la web tras la comprobacin
			Identificador.testIdentificadores(identificadores, "Identificador", estado,	
					request, "");
		
			// Comprueba los botones de radio de respuesta correcta que uno est marcado
			String respCorrecta = request.getParameter("Correcto");
			if (respCorrecta == null)
				estado.aadeError("Debe marcar una respuesta como correcta<br />", 
						"Identificador0");
			
			//Lee los checkbox de respuesta en posicin fija
			boolean[] respFija = new boolean[numOpc];
			Comprobaciones.leeCheckBoxes(respFija, "Fija", request);
		
			//Si todo est bien llama al mtodo que finaliza el tem y redirecciona al cliente
			if (estado.isEstado() == true){
				terminaInlinechoice(request, response, opciones, identificadores, 
						respCorrecta, respFija, texto1, texto2);
			} else {
				// Si ha habido algn fallo pone los datos que ya ha introducido el usuario 
				// junto a los mensajes de aviso y el foco en el primer fallo encontrado
				request.setAttribute("Opciones",opciones);
				request.setAttribute("Identificador",identificadores);
				request.setAttribute("Correcto",respCorrecta);
				request.setAttribute("Fija",respFija);
				request.setAttribute("Texto1", texto1);
				request.setAttribute("Texto2", texto2);
				request.setAttribute("Aviso",estado.getMensaje());
				request.setAttribute("Focus",estado.getFocusON());
			}
		} else {
			// Si es la primera vez que se visita la pgina pone el foco en el primer campo
			request.setAttribute("Focus", "Texto1");
			// Inicializa los identificadores a unos valores por defecto
			for (int i = 0; i < identificadores.length; i++)
				identificadores[i] = new Identificador(identificadorBase + (i+1));
			request.setAttribute("Identificador",identificadores);
		}
		
		// El nmero de respuestas y si se mezclan las respuestas aleatoriamente hacen falta 
		// siempre, sea la primera vez que visita la pgina o no y pone el foco en el primer 
		// campo del formulario
		request.setAttribute("Num_Opc",num_Opc);
		request.setAttribute("Shuff",shuff);
		
		return true;
	}
	/**
	 * Este mtodo se ejecuta una vez que se comprueba que todos los parmetros son correctos. 
	 * Usa el objeto <code>InlinechoiceXML</code> creado anteriormente y almacenado en sesin 
	 * para guardar el resto de los datos asociados a la pregunta, redireccionando la 
	 * respuesta a la pgina "seleccionasignatura.jsp" para que el usuario seleccione una 
	 * asignatura en la que guardar el tem XML. Si el objeto <code>InlinechoiceXML</code> no 
	 * se ha podido recuperar de sesin porque sta ha expirado, redirecciona la respuesta a 
	 * una pgina para informar al usuario.
	 * 
	 * @param 	req				peticin del cliente para acceder a la sesin
	 * @param 	res				respuesta para el cliente para redireccionarlo a otra pgina
	 * @param 	respuestas		array de string con cada una de las respuestas
	 * @param 	identif			array de identificador con los identificadores de las 
	 * 							respuestas
	 * @param 	correct			string con el nmero de la opcin que es correcta
	 * @param 	fija			array de boolean marcando las respuestas que son fijas
	 * @param	texto1			string con el texto antes de las opciones
	 * @param	texto2			string con el texto despus de las opciones
	 * @throws 	IOException		si hay algn error al redireccionar la respuesta
	 * @see		xml.items.InlinechoiceXML
	 * @see		javax.servlet.http.HttpSession
	 * @see		javax.servlet.http.HttpServletResponse#sendRedirect(java.lang.String)
	 */
	private void terminaInlinechoice(HttpServletRequest req, HttpServletResponse res, 
			String[] respuestas, Identificador[] identif, String correct, boolean[] fija, 
			String texto1, String texto2) throws IOException{
		HttpSession sesion = req.getSession();
		// Si el objeto sesin no es nuevo la sesin no ha expirado
		if (!sesion.isNew()){	
			// Rellena los campos que faltan en el objeto ChoiceXML que se cre en la 
			// primera parte de la toma de datos
			InlinechoiceXML icXML = (InlinechoiceXML)sesion.getAttribute("InlineChoice");
			icXML.setRespuestas(respuestas);
			icXML.setIdentificadores(identif);
			icXML.setCorrecto(Integer.parseInt(correct));
			icXML.setFija(fija);
			icXML.setTextos(texto1, texto2);
			// Almacena el tem modificado en sesin para utilizarlo ms adelante
			sesion.setAttribute("AItem", icXML);
			// Redirecciona la respuesta a "seleccionasignatura.jsp" indicndole la direccin 
			// a la que debe ir despus
			res.sendRedirect("seleccionasignatura.jsp?Destino=finitem.jsp");
		} else			// Si la sesin ha expirado
			res.sendRedirect("errorfinsesion.jsp");

	}
}