package control;

import utilidades.*;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.*;

import xml.items.MatchXML;

/**
 * Componente de control implcito asociado a la pgina JSP "respuestasmatch.jsp". Es la 
 * lgica asociada a esa pgina JSP. Se ejecutar su mtodo <code>doLogic</code> siempre que 
 * sea llamada la pgina. Su funcin es comprobar que todos los parmetros finales de la toma 
 * de datos de una pregunta tipo "Match" sean correctos, en cuyo caso redirecciona la respuesta 
 * a la pgina "seleccionasignatura.jsp" para elegir donde guardar el tem. La clase implementa 
 * la interfaz <code>Control</code>, implementando su mtodo <code>doLogic</code>, que es el 
 * encargado de realizar la lgica asociada a la pgina "respuestasmatch.jsp". Tiene otro 
 * mtodo que simplemente se ejecuta si los parmetros son correctos para redireccionar la 
 * respuesta y terminar de crear el objeto <code>MatchXML</code>.
 * 
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Respuestasmatch implements Control {

	/**
	 * Realiza la comprobacin de los parmetros que se recogen en la pgina 
	 * "respuestasmatch.jsp". Ser llamado por el filtro <code>ControlFilter</code> en el caso 
	 * de que detecte que la pgina requerida es "respuestasmatch.jsp". Ya que es llamado 
	 * desde un filtro, ser ejecutado siempre que llegue una peticin de un cliente y antes de 
	 * que la peticin llegue al Servlet (pgina JSP compilada a Servlet) que sirve la 
	 * respuesta al cliente. Comprueba uno por uno todos los parmetros, y, en caso de que sean 
	 * correctos,  llama al mtodo <code>terminaMatch</code> para terminar de crear el objeto 
	 * del tem y redireccionar la respuesta.  
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se enve la respuesta correctamente
	 * @throws	IOException			no debe de producirse
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 * @see		utilidades.EstadoWeb
	 * @see		utilidades.EnteroPositivo
	 * @see		utilidades.Identificador
	 * @see		utilidades.Comprobaciones
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		// Nombre base para los identificadores de las opciones del grupo 2 por defecto
		final String identificadorBaseGrupo2 = "Grupo2_";
		request.setAttribute("IdentificadorBaseGrupo2", identificadorBaseGrupo2);
		// Nombre base para los identificadores de las opciones del grupo 1 por defecto
		final String identificadorBaseGrupo1 = "Grupo1_";
		request.setAttribute("IdentificadorBaseGrupo1", identificadorBaseGrupo1);
				
		// Estos elementos siempre estarn presentes en request al venir de la pgina anterior
		// Nmero de respuestas del grupo 1
		String numGrupo1= request.getParameter("NumGrupo1");
		// Nmero de respuestas del grupo 2
		String numGrupo2= request.getParameter("NumGrupo2");
		// Mezclar respuestas s o no
		String shuff = request.getParameter("Shuffle");
		int numGrup1 = Integer.parseInt(numGrupo1);		// Nmeros de respuestas
		int numGrup2 = Integer.parseInt(numGrupo2);		// de cada grupo en un entero
		// Arrays con los identificadores del grupo 1 y del grupo 2
		Identificador[] identificadores1 = new Identificador[numGrup1];
		Identificador[] identificadores2 = new Identificador[numGrup2];
			
		//Comprueba si es la primera vez que se entra en la pgina
		if(request.getParameter("Form") != null){
		
			// Arrays con las respuestas del grupo 1 y del grupo 2
			String[] respuestas1 = new String [numGrup1];
			String[] respuestas2 = new String [numGrup2];

			// Estado de la peticin web. Almacena si son correctos los parmetros, los 
			// mensajes de aviso al usuario y dnde colocar el foco en la pgina
			EstadoWeb estado = new EstadoWeb();		
			
			int i;
			// Lee las respuestas del grupo 1, y si alguna est vaca aade el cdigo de error
			for(i = 0; i < respuestas1.length; i++){
				respuestas1[i] = request.getParameter("Grupo1_"+i);
				if (respuestas1[i] == null || respuestas1[i].equals(""))
					estado.aadeError("Debe rellenar la respuesta " + (i+1) + " del grupo 1" +
							"<br />", "Grupo1_" + i);
			}
			// Lee las respuestas del grupo2, y si alguna est vaca aade el cdigo de error
			for(i = 0; i < respuestas2.length; i++){
				respuestas2[i] = request.getParameter("Grupo2_"+i);
				if (respuestas2[i] == null || respuestas2[i].equals(""))
					estado.aadeError("Debe rellenar la respuesta " + (i+1) + " del grupo 2" +
							"<br />", "Grupo2_" + i);
			}
			
			// Lee los identificadores del grupo 1 comprobando que sean correctos y no haya 
			// ninguno repetido, devolviendo el estado de la web tras la comprobacin
			Identificador.testIdentificadores(identificadores1, "Identificador1_", estado, 
					request, "En el grupo 1 ");
			
			// Lee los identificadores del grupo 2 comprobando que sean correctos y no haya 
			// ninguno repetido, devolviendo el estado de la web tras la comprobacin
			Identificador.testIdentificadores(identificadores2, "Identificador2_", estado, 
					request, "En el grupo 2 ");
			
			// Comprueba los checkbox de respuesta correcta de cada fila que al menos uno est 
			// marcado en cada fila y guarda el nmero total de correctas en cada fila
			// Nmero de respuesas correctas en cada fila
			int[] numCorrectas1 = new int[numGrup1];
			// Matriz de combinaciones correctas
			boolean[][] respCorrecta = new boolean[numGrup1][numGrup2];
			for (i = 0; i < numGrup1; i++){
				numCorrectas1[i] = Comprobaciones.sumaCorrectas(respCorrecta[i], "Correcto" + 
						i + "_", request);
				if  (numCorrectas1[i] == 0)		// Si el nmero de correctas de la fila es cero
					estado.aadeError("Error: no ha marcado ninguna respuesta correcta en " +
							"la fila " + (i+1) + "<br />", "Correcto" + i + "_0");
			}
			// Suma el nmero de elementos correctos en cada columna para comprobar los 
			// nmeros mximos de eleccin del grupo 2
			int[] numCorrectas2 = new int[numGrup2];	// Nmero de correctas en cada columna
			for (int j = 0; j < numGrup2; j++){
				numCorrectas2[j] = 0;		// Inicializa el nmero de correctas en la columna
				for (i = 0; i < numGrup1; i++)
					if (respCorrecta[i][j])
						numCorrectas2[j]++;
			}
			
			
			// Recoge el valor del nmero mximo de elecciones comprobando que sea vlido
			// Valores del nmero mximo de elecciones de cada elemento del grupo 1
			EnteroPositivo[] maxChoices1 = new EnteroPositivo[numGrup1];  
			// Valores del nmero mximo de elecciones de cada elemento del grupo 2
			EnteroPositivo[] maxChoices2 = new EnteroPositivo[numGrup2];		
			
			// Lee los valores de los nmeros mximos de elecciones introducidos en la pgina
			// Valores introducidos para el grupo 1
			String[] max1 = new String[maxChoices1.length];
			for (i = 0; i < maxChoices1.length; i++)
				max1[i] = request.getParameter("Max1_" + i);
			// Valores introducidos para el grupo 2
			String[] max2 = new String[maxChoices2.length];
			for (i = 0; i < maxChoices2.length; i++)
				max2[i] = request.getParameter("Max2_" + i);
			// Comprueba el valor introducido para el grupo 1 que sea mayor que el nmero de 
			// respuestas correctas o cero, y menor que el nmero total de respuestas posibles
			Comprobaciones.testTablaMaximos(maxChoices1, "Max1_", estado, max1, "del grupo 1", 
					"fila", numCorrectas1, numGrup2);
			// Comprueba el valor introducido para el grupo 2 que sea mayor que el nmero de 
			// respuestas correctas o cero, y menor que el nmero total de respuestas posibles
			Comprobaciones.testTablaMaximos(maxChoices2, "Max2_", estado, max2, "del grupo 2", 
					"columna", numCorrectas2, numGrup1);
			
			// Nmero mximo de elecciones totales
			String max = request.getParameter("NumMax");
			// Comprueba el valor introducido para el total que sea mayor que el nmero de 
			// respuestas correctas o cero, y menor que el nmero total de respuestas posibles
			EnteroPositivo  numMax = new EnteroPositivo(max);
			int totalCorrectas = 0;				// Total de respuestas marcadas como correctas
			for (i = 0; i < numGrup1; i++)
				totalCorrectas += numCorrectas1[i];
			Comprobaciones.testMaximo(numMax, estado, "NumMax", totalCorrectas, 
					numGrup1 * numGrup2);
			
			//Lee los checkbox de respuesta en posicin fija
			boolean[] respFija1 = new boolean[numGrup1];	// Respuestas fijas del grupo 1
			boolean[] respFija2 = new boolean[numGrup2];	// Respuestas fijas del grupo 2
			Comprobaciones.leeCheckBoxes(respFija1, "Fija1_",request);
			Comprobaciones.leeCheckBoxes(respFija2, "Fija2_", request);
		
			//Si todo est bien llama al mtodo que finaliza el tem y redirecciona al cliente
			if (estado.isEstado() == true){
				terminaMatch(request, response, respuestas1, respuestas2, identificadores1, 
						identificadores2, numMax, maxChoices1, maxChoices2, respCorrecta, 
						respFija1, respFija2);
			} else {
				// Si ha habido algn fallo pone los datos que ya ha introducido el usuario 
				// junto a los mensajes de aviso y el foco en el primer fallo encontrado
				request.setAttribute("Grupo1", respuestas1);
				request.setAttribute("Grupo2", respuestas2);
				request.setAttribute("Identificador1", identificadores1);
				request.setAttribute("Identificador2", identificadores2);
				request.setAttribute("NumMax", max);
				request.setAttribute("Max1", max1);
				request.setAttribute("Max2", max2);
				request.setAttribute("Correcto", respCorrecta);
				request.setAttribute("Fija1", respFija1);
				request.setAttribute("Fija2", respFija2);
				request.setAttribute("Aviso", estado.getMensaje());
				request.setAttribute("Focus", estado.getFocusON());
			}
		} else {
			// Si es la primera vez que se visita la pgina pone el foco en el primer campo
			request.setAttribute("Focus", "NumMax");
			// Inicializa los identificadores de los dos grupos a unos valores por defecto
			for (int i = 0; i < identificadores1.length; i++)
				identificadores1[i] = new Identificador(identificadorBaseGrupo1 + (i+1));
			for (int i = 0; i < identificadores2.length; i++)
				identificadores2[i] = new Identificador(identificadorBaseGrupo2 + (i+1));
			request.setAttribute("Identificador1", identificadores1);
			request.setAttribute("Identificador2", identificadores2);
		}
		
		// El nmero de respuestas en cada grupo y si se mezclan las respuestas aleatoriamente 
		// hacen falta siempre, sea la primera vez que visita la pgina o no y pone el foco en 
		// el primer campo del formulario
		request.setAttribute("NumGrupo1",numGrupo1);
		request.setAttribute("NumGrupo2",numGrupo2);
		request.setAttribute("Shuffle",shuff);

		return true;
	}
	/**
	 * Este mtodo se ejecuta una vez que se comprueba que todos los parmetros son correctos. 
	 * Usa el objeto <code>MatchXML</code> creado anteriormente y almacenado en sesin para 
	 * guardar el resto de los datos asociados a la pregunta, redireccionando la respuesta a 
	 * la pgina "seleccionasignatura.jsp" para que el usuario seleccione una asignatura en la 
	 * que guardar el tem XML. Si el objeto <code>MatchXML</code> no se ha podido recuperar 
	 * de sesin porque sta ha expirado, redirecciona la respuesta a una pgina para informar 
	 * al usuario.
	 * 
	 * @param 	req				peticin del cliente para acceder a la sesin
	 * @param 	res				respuesta para el cliente para redireccionarlo a otra pgina
	 * @param 	resp1			array de string con cada una de las opciones del grupo 1
	 * @param 	resp2			array de string con cada una de las opciones del grupo 2
	 * @param 	identif1		array de identificador con los identificadores del grupo 1
	 * @param 	identif2		array de identificador con los identificadores del grupo 2
	 * @param 	max				enteroPositivo con el nmero mximo de elecciones totales
	 * @param 	max1			array de enteroPositivo con el nmero mximo de elecciones de 
	 * 							cada elemento del grupo 1
	 * @param 	max2			array de enteroPositivo con el nmero mximo de elecciones de 
	 * 							cada elemento del grupo 2
	 * @param 	correct			array de boolean de dos dimensiones con las combinaciones 
	 * 							entre elementos del grupo 1 y del 2 que son correctas
	 * @param 	fija1			array de boolean indicando las opciones del grupo 1 fijas
	 * @param 	fija2			array de boolean indicando las opciones del grupo 2 fijas
	 * @throws 	IOException		si hay algn error al redireccionar la respuesta
	 * @see		xml.items.MatchXML
	 * @see		javax.servlet.http.HttpSession
	 * @see		javax.servlet.http.HttpServletResponse#sendRedirect(java.lang.String)
	 */
	private void terminaMatch(HttpServletRequest req, HttpServletResponse res, 
			String[] resp1, String[] resp2, Identificador[] identif1, Identificador[] identif2,
			EnteroPositivo max, EnteroPositivo[] max1, EnteroPositivo[] max2, 
			boolean[][] correct, boolean[] fija1, boolean[] fija2) throws IOException{
		HttpSession sesion = req.getSession();
		// Si el objeto sesin no es nuevo la sesin no ha expirado
		if (!sesion.isNew()){	
			// Rellena los campos que faltan en el objeto ChoiceXML que se cre en la 
			// primera parte de la toma de datos
			MatchXML maXML = (MatchXML)sesion.getAttribute("Match");
			maXML.setRespuestas1(resp1);
			maXML.setRespuestas2(resp2);
			maXML.setIdentificadores1(identif1);
			maXML.setIdentificadores2(identif2);
			maXML.setMaxChoices(max.getEnteroPositivo());
			maXML.setMax1(max1);
			maXML.setMax2(max2);
			maXML.setRespCorrecta(correct);
			maXML.setFija1(fija1);
			maXML.setFija2(fija2);
			// Almacena el tem modificado en sesin para utilizarlo ms adelante
			sesion.setAttribute("AItem", maXML);
			// Redirecciona la respuesta a "seleccionasignatura.jsp" indicndole la direccin 
			// a la que debe ir despus
			res.sendRedirect("seleccionasignatura.jsp?Destino=finitem.jsp");
		} else 			// Si la sesin ha expirado
			res.sendRedirect("errorfinsesion.jsp");
		
	}
}