package control;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import utilidades.ListaArchivos;

/**
 * Componente de control implcito asociado a la pgina JSP "seleccionasignatura.jsp". Es la 
 * lgica asociada a esa pgina JSP. Su mtodo <code>doLogic</code> se ejecutar siempre que 
 * sea llamada la pgina. Se encarga de pedir al usuario una asignatura de trabajo.
 *  
 * @author 	David Domnguez
 * @see		control.Control
 * 
 */
public class Seleccionasignatura implements Control {

	/**
	 * Ser llamado por el filtro <code>ControlFilter</code>" en el caso de que detecte que la 
	 * pgina requerida es "seleccionasignatura.jsp". Ya que es llamado desde un filtro, ser 
	 * ejecutado siempre que llegue una peticin de un cliente y antes de que la peticin 
	 * llegue al Servlet (pgina JSP compilada a Servlet) que sirve la respuesta al cliente. 
	 * Pide al usuario que seleccione una asignatura en la que realizar las acciones, y le 
	 * permite crear nuevos directorios de asignaturas. Si todo es correcto permite seleccionar 
	 * una asignatura e ir a la pgina que indique el parmetro recibido en request como <code>
	 * Destino</code>. 
	 * 
	 * @param	request				la peticin http enviada por el cliente
	 * @param	response			la respuesta http a enviar desde el servidor
	 * @return						true para que se sigan ejecutando los dems filtros del 
	 * 								servidor y se le enve la respuesta correctamente
	 * @throws	IOException			si hay algn problema con el acceso a los directorios de 
	 * 								asignatura
	 * @throws	ServletException	si hay algn problema accediendo a request o response
	 * @see 	control.Control#doLogic(javax.servlet.http.HttpServletRequest,javax.servlet.http.HttpServletResponse)
	 * @see		javax.servlet.http.HttpServletRequest
	 * @see		javax.servlet.http.HttpServletResponse
	 * @see 	control.ControlFilter
	 * @see		java.io.File
	 * @see		java.util.Vector
	 * @see		utilidades.ListaArchivos#ListaDirectorios(File)
	 * 
	 */
	public boolean doLogic(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {
		
		// Aviso recibido de otra pgina a mostrar en esta
		String aviso = request.getParameter("Aviso");
		// Informacion recibida de otra pgina a mostrar en esta
		String informacion = request.getParameter("Informacion");
		ServletContext sc =	request.getSession().getServletContext();
		// Directorio inicial de trabajo (guardado en "web.xml")
		String dirTrabajo = sc.getInitParameter("Directorio Trabajo");
		// Pgina a la que ir una vez seleccionada la asignatura
		String destino = request.getParameter("Destino");

		// El directorio de las asignaturas es el directorio base de trabajo
		File dirAsignaturas = new File(sc.getRealPath(dirTrabajo));
		
		// Si no existe el directorio de trabajo lo intenta crear
		if (!dirAsignaturas.exists())
			if (!dirAsignaturas.mkdir())
				throw new IOException("Error: No se puede crear el directorio base de " +
						"trabajo");
		
		// Obtiene una lista de los nombre de las asignaturas y lo guarda en request
		Vector /* String */ dirNames = ListaArchivos.ListaDirectorios(dirAsignaturas);
		request.setAttribute("NombresAsignaturas",dirNames);
		
		// Almacena en request el nombre del directorio de trabajo y de la pgina a la que ir 
		// despus
		request.setAttribute("DirectorioTrabajo", dirTrabajo);
		request.setAttribute("Destino", destino);

		// Avisos e informacin para el usuario de la pgina
		request.setAttribute("Aviso", aviso);
		request.setAttribute("Informacion", informacion);
		
		return true;
	}
}