package utilidades;

import javax.servlet.http.HttpServletRequest;

/**
 * Clase con diversos mtodos estticos para leer los datos introducidos en las pginas de 
 * tomas de datos de los tems y el test y comprobar, en aquellos que haga falta, que sean 
 * correctos. 
 * 
 * @author David Domnguez
 *
 */
public class Comprobaciones{

	/**
	 * Mtodo para ayudar a leer un conjunto de checkboxes con un nombre base comn de una 
	 * pgina Web informando del nmero que estn marcadas. Lee los checkboxes recogidos en una 
	 * pgina web con nombre base comn, y rellena un <code> array</code> de boolean con las 
	 * respuestas correctas, devolviendo el nmero de respuestas correctas totales.
	 * 
	 * @param 	respCorrecta	array de boolean a rellenar marcando las respuestas correctas
	 * @param 	origen			nombre base de los checkboxes a leer
	 * @param 	request			peticin recibida del cliente donde estn almacenados las 
	 * 							respuestas correctas marcadas por el usuario
	 * @return					entero con el nmero total de respuestas marcadas correctas
	 * 
	 * @see		javax.servlet.http.HttpServletRequest#getParameter
	 * @see		utilidades.EstadoWeb#aadeError(String, String)
	 */
	public static int sumaCorrectas (boolean[] respCorrecta, String origen, 
			HttpServletRequest request){
		
		int numCorrectas = 0;		// Inicializa el nmero de respuestas correctas a cero
		String check;				// Variable intermedia para leer el valor de los checkboxes
		
		for (int i = 0; i < respCorrecta.length; i++){
			check = request.getParameter(origen + i);		// Lee el checkbox i
			if (check != null && check.equalsIgnoreCase("ON")){
				// Si el checkbox es igual a "ON" la respuesta es correcta y se incrementa el 
				// nmero de respuestas correctas
				respCorrecta[i] = true;
				numCorrectas++;
			}
			else
				// Si no es igual a "ON", respuesta incorrecta
				respCorrecta[i] = false;
		}
		

		return numCorrectas;
	}
	
	/**
	 * Mtodo para ayudar a leer un conjunto de checkboxes con un nombre base comn de una 
	 * pgina Web. Lee un conjunto de checkboxes inicializando la tabla de <code>boolean</code> 
	 * que recibe como parmetro en funcin de si est cada checkbox marcado o no.
	 * 
	 * @param	respFija	array de boolean con las respuestas fijas
	 * @param 	origen		nombre base de los checkbox de respuesta fija
	 * @param 	request		peticin del cliente
	 * @see		javax.servlet.http.HttpServletRequest#getParameter
	 */
	public static void leeCheckBoxes(boolean[] respFija, String origen, 
			HttpServletRequest request){
		String checkValor;
		for (int i = 0; i < respFija.length; i++){
			checkValor = request.getParameter(origen + i);
			if (checkValor != null && checkValor.equalsIgnoreCase("ON"))
				respFija[i] = true;
			else
				respFija[i] = false;
		}

	}
	
	/**
	 * Comprueba que el nmero mximo de elecciones introducido sea correcto en preguntas con 
	 * un solo grupo de respuestas y entre las que se puede seleccionar ms de una como 
	 * correcta. Comprueba que el valor introducido sea un entero positivo correcto mediante 
	 * el mtodo <code>esPositivo</code> de <code>EnteroPositivo</code>, mayor que el nmero 
	 * de opciones marcadas como correctas o cero, y menor que el nmero total de opciones de 
	 * respuesta disponibles.
	 * 
	 * @param 	maxChoices		enteroPositivo inicializado con el string introducido por el 
	 * 							usuario con el nmero mximo de elecciones a comprobar
	 * @param 	estado			estadoWeb, representando el estado de la web. Se modifica con 
	 * 							el estado tras la comprobacin del nmero mximo de elecciones.
	 * @param 	campo			campo de la web de donde se ha ledo el nmero mximo de 
	 * 							elecciones indicando dnde poner el foco en caso de error
	 * @param 	numCorrectas	nmero de respuestas marcadas como correctas totales
	 * @param 	numMaxOpciones	nmero mximo de respuestas posibles que tiene el candidato 
	 * 							para elegir
	 * @see		utilidades.MensajeEstado
	 * @see		utilidades.EstadoWeb#aadeError(String, String)
	 * @see		utilidades.EnteroPositivo
	 */
	public static void testMaximo(EnteroPositivo maxChoices, EstadoWeb estado, String campo, 
			int numCorrectas, int numMaxOpciones){

		MensajeEstado msg = maxChoices.esPositivo();
		if (!msg.isEstado())		// Si el nmero no es un entero positivo vlido
			estado.aadeError("Error: el nmero mximo de elecciones " + msg.getMensaje() + 
					"<br />", campo);
		else if (maxChoices.getEnteroPositivo() != 0 &&	
				maxChoices.getEnteroPositivo() < numCorrectas)
			estado.aadeError("El nmero mximo de elecciones debe ser mayor o igual que el " +
					"nmero de respuestas correctas o cero", campo);
		else if (maxChoices.getEnteroPositivo() > numMaxOpciones)
			estado.aadeError("El nmero mximo de elecciones debe ser menor o igual que el " +
					"nmero de respuestas posibles<br />", campo);
	}
	
	/**
	 * Comprueba los nmeros mximos de elecciones introducidos para una pregunta en la que 
	 * hay varios grupos de opciones, y donde hay al menos un conjunto de elementos en el que 
	 * cada elemento tiene su propio nmero mximo de elecciones con elementos de otro grupo. 
	 * Comprueba todos los elementos de un grupo uno por uno, examinando que el valor 
	 * introducido sea un entero positivo correcto mediante el mtodo <code>esPositivo</code> 
	 * de <code>EnteroPositivo</code>, mayor que el nmero de opciones marcadas como correctas 
	 * o cero, y menor que el nmero total de opciones de respuesta disponibles.
	 * 
	 * @param 	maxChoices		array de enteroPositivo con el nmero mximo de elecciones de 
	 * 							cada elemento del grupo a rellenar con los string ledos de la
	 * 							pgina web contenidos en el parmetro <code>max</code>
	 * @param 	campo			nombre base del campo de la web de donde se ha ledo el nmero 
	 * 							mximo de elecciones y dnde poner el foco en caso de error
	 * @param 	estado			estadoWeb, representando el estado de la web. Se modifica con 
	 * 							el estado tras las comprobaciones
	 * @param 	max				array de string con los nmero mximos de elecciones ledos de 
	 * 							la pgina web
	 * @param 	grupo			string con el grupo al que pertenecen los nmeros mximos de 
	 * 							elecciones.	Para rellenar correctamente los avisos al usuario.
	 * @param 	elemento		Si tratamos a las filas o las columnas. Para rellenar 
	 * 							correctamente los avisos al usuario en caso de error.
	 * @param 	numCorrectas	array con el nmero de respuestas marcadas como correctas de 
	 * 							cada elemento
	 * @param 	numOpciones		nmero mximo de opciones posibles a elegir como correctas
	 * @see		utilidades.MensajeEstado
	 * @see		utilidades.EstadoWeb#aadeError(String, String)
	 * @see		utilidades.EnteroPositivo
	 */
	public static void testTablaMaximos(EnteroPositivo[] maxChoices, String campo, 
				EstadoWeb estado, String[] max, String grupo, String elemento, 
				int[] numCorrectas, int numOpciones){
		
		// Cadena con el nombre de los elementos opuestos
		String elemOpuesto;
		if (elemento.equals("fila"))
				elemOpuesto = "columna";
		else
				elemOpuesto = "fila";

		for (int i = 0; i < maxChoices.length; i++){
			maxChoices[i] = new EnteroPositivo(max[i]);
			// Comprueba el nmero de elecciones, que sea un entero correcto, que no sea menor 
			// que el nmero de elementos marcados si es distinto de cero y que no sea mayor 
			// que el nmero de elementos disponibles
			MensajeEstado msg = maxChoices[i].esPositivo();
			if (!msg.isEstado())		// Si el nmero no es un entero positivo vlido
				estado.aadeError("Error: el nmero mximo de elecciones de la " + elemento +
						" " + (i+1) + " " + msg.getMensaje() + "<br />", campo + i);
			else if (maxChoices[i].getEnteroPositivo() != 0 &&	
					maxChoices[i].getEnteroPositivo() < numCorrectas[i])
				estado.aadeError("El nmero mximo de elecciones de la " + elemento + " " + 
						(i+1) +	" " + grupo + " debe ser mayor o igual que el nmero de " +
						"respuestas " +	"correctas en esa " + elemento + " o cero<br />", 
						campo+ i);
			else if (maxChoices[i].getEnteroPositivo() > numOpciones)
				estado.aadeError("El nmero mximo de elecciones de la " + elemento + " " + 
						(i+1) +	" " + grupo + " debe ser menor que el nmero de " + 
						elemOpuesto + "s<br />", 
						campo + i);
		}

	}
}