package utilidades;

/**
 * Clase que contiene la representacin de un nmero entero positivo. Se encarga de recibirlo 
 * como un <code>String</code> y de comprobar que sea efectivamente un nmero entero positivo. 
 * Contiene mtodos para almacenar y recoger el nmero, e informa de cualquier anomala en l.
 * 
 * @author David Domnguez
 *
 */
public class EnteroPositivo {
	/**
	 * Variable que contiene el nmero en un String.
	 */
	private String stringNumero = null;
	/**
	 * Variable con el nmero ya en un entero.
	 */
	private int enteroPositivo;

	/**
	 * Constructor que recibe la representacin del nmero en un <code>String</code>. Lo 
	 * almacena en la variable de clase dedicada a tal efecto.
	 * 
	 * @param 	stringNumero	nmero a almacenar en la clase como un String.
	 */
	public EnteroPositivo(String stringNumero) {
		super();
		this.stringNumero = stringNumero;
	}

	/**
	 * Mtodo "get" para el nmero entero positivo como un entero. 
	 * ATENCIN: Antes de usarlo se debe haber comprobado que el nmero es un entero realmente 
	 * mediante el mtodo <code>esPositivo</code>, y en caso de que devuelva un <code>
	 * MensajeEstado</code> que indique validez, podr usarse este mtodo.
	 * 
	 * @return		entero representando el valor que almacena la clase
	 */
	public int getEnteroPositivo() {
		return enteroPositivo;
	}

	/**
	 * Mtodo "get" para el nmero entero positivo almacenado como un <code>String</code>. 
	 * Se puede usar siempre, pero sin usar el mtodo <code>esPositivo</code> no se tendr 
	 * seguridad de que el <code>String</code> represente un nmero entero positivo vlido.
	 * 
	 * @return		String con el valor almacenado en la clase
	 */
	public String getStringNumero() {
		return stringNumero;
	}

	/**
	 * Comprueba que el nmero <code>String</code> almacenado sea un nmero entero positivo. 
	 * Comprueba que no sea una cadena vaca, que sea un nmero entero, y que sea cero o mayor 
	 * que cero, es decir, un nmero entero positivo vlido.
	 * 
	 * @return		mensajeEstado indicando si ha ido bien la comprobacin, en cuyo caso no 
	 * 				incluye	mensajes, o, si ha ido mal, indicando en el mensaje el error 
	 * 				ocurrido en la comprobacin
	 * @see			utilidades.MensajeEstado
	 * @see			Integer#parseInt(java.lang.String)
	 */
	public MensajeEstado esPositivo(){
		MensajeEstado msg = new MensajeEstado();	// Mensaje de salida con el estado final
		
		if (stringNumero != null && !stringNumero.equals("")){
			// Si la cadena no est vaca
			try {
				enteroPositivo = Integer.parseInt(stringNumero);
				if (enteroPositivo < 0){
					// Si el nmero es menor que cero (negativo)
					msg.setMensaje("es un nmero menor que cero");
					msg.setEstado(false);
				}
			} catch (NumberFormatException e){
				// Si la cadena no contiene un nmero entero vlido captura la excepcin
				msg.setMensaje("es un entero positivo no vlido");
				msg.setEstado(false);
			}
		} else {
			// Si la cadena est vaca
			msg.setMensaje("est vaco");
			msg.setEstado(false);
		}
		
		// Devuelve el mensaje con el estado de la comprobacin
		return msg;
	}
}