package utilidades;

/**
 * Clase que representa el estado de comprobacin de una Web. La clase hereda de <code>
 * MensajeEstado</code>, aadindole la funcionalidad de almacenar adems en qu componente del 
 * formulario de la pgina web se debe localizar el foco en caso de error. Se encarga de 
 * almacenar el estado de la comprobacin de los datos en cualquier toma de datos de creacin de 
 * un tem o un test, y, en caso de error, contiene los mensajes de aviso a mostrar al usuario y 
 * dnde posicionar el foco en la pgina.
 * 
 * @author	David Domnguez
 * @see		utilidades.MensajeEstado
*
 */
public class EstadoWeb extends MensajeEstado {

	/**
	 * Nombre del campo errneo sobre el que ir el foco en la pgina.
	 */
	protected String focusON;

	/**
	 * El constructor llama al constructor de la superclase e inicializa la posicin del foco 
	 * a null.
	 * 
	 * @see		utilidades.MensajeEstado
	 */
	public EstadoWeb() {
		super();
		focusON = null;
	}

	/**
	 * Mtodo "get" para obtener la posicin del foco. Obtiene el <code>String</code> que lo 
	 * almacena.
	 * 
	 * @return	string con el nombre del campo del formulario en el que localizar el foco en 
	 * 			caso de	error
	 */
	public String getFocusON() {
		return focusON;
	}

	/**
	 * Aade un error al estado de la comprobacin. Al aadir el error se aade el nuevo aviso 
	 * al ya existente, se pone el estado en falso, ya que aade un error, y, si el foco an 
	 * no se ha inicializado a ningn valor, lo inicializa al que recibe como parmetro. Con 
	 * esto logra que foco se posicione en el primer error encontrado y no vare al aadir 
	 * ms errores.
	 *  
	 * @param 	aviso	mensaje de aviso a aadir a los existentes
	 * @param 	focus	campo errneo en el que posicionar el foco en la pgina
	 * @see		utilidades.MensajeEstado
	 */
	public void aadeError(String aviso, String focus){
		this.mensaje += aviso;			// Aade el nuevo aviso a los ya existentes
		this.estado = false;			// El estado es incorrecto ya que se aade un error
		if (this.focusON == null)		// Si el foco an no se ha inicializado se inicializa 
			this.focusON = focus;		// al valor recibido en el parmetro
	}
}