package utilidades;

/**
 * Clase que contiene un estado y un mensaje. Se utiliza como salida de muchas comprobaciones, 
 * avisando de si es correcta y de qu ha fallado en su caso.
 * 
 * @author David Domnguez
 *
 */
public class MensajeEstado {
	
	/**
	 * Mensaje a almacenar en caso de que haya error.
	 */
	protected String mensaje;
	/**
	 * Estado de la comprobacin.
	 */
	protected boolean estado;
	
	/**
	 * Constructor que simplemente pone el estado correcto, inicializando el estado a true. 
	 * Si no lo fuera, deber ponerse a falso posteriormente. El mensaje es un <code>String
	 * </code> que se inicializa a la cadena vaca para despus ir aadiendo avisos sin 
	 * problemas de si es la primera vez o no que se aade una cadena, aadiendo las sucesivas 
	 * con el operador '+' para cadenas.
	 */
	public MensajeEstado() {
		super();
		estado = true;				// Estado correcto
		mensaje = "";				// Cadena vaca
	}
	
	/**
	 * Mtodo para obtener el estado de la clase.
	 * 
	 * @return				boolean con el estado de la clase
	 */
	public boolean isEstado() {
		return estado;
	}
	
	/**
	 * Mtodo "set" para establecer el estado de la clase.
	 * 
	 * @param	estado		el estado al que poner la clase
	 */
	public void setEstado(boolean estado) {
		this.estado = estado;
	}
	
	/**
	 * Mtodo "get" para obtener el mensaje de la clase.
	 * 
	 * @return				el mensaje almacenado en la clase
	 * @see		#setMensaje
	 */
	public String getMensaje() {
		return mensaje;
	}
	
	/**
	 * Mtodo "set" para establecer el mensaje.
	 * 
	 * @param	mensaje		string con el mensaje a aadir al mensaje actual
	 * @see		#getMensaje
	 */
	public void setMensaje(String mensaje) {
		this.mensaje += mensaje;
	}
}