package utilidades;

import java.io.File;

/**
 * Clase que representa un resumen de los tems (archivos XML independientes que representan al 
 * tem). El resumen del tem contiene el nombre del tem (nombre del archivo XML) en un <code>
 * String</code>, el ttulo del tem en otro <code>String</code>, y el objeto <code>File</code> 
 * que representa al archivo XML del tem. Contiene un constructor que inicializa todos sus 
 * valores, mtodos "getters" para obtenerlos, e implementa la interfaz <code>Comparable</code> 
 * para permitir que se ordene una coleccin de objetos <code>ResumenItem</code>.
 * 
 * @author 	David Domnguez
 * @see		Comparable
 */
public class ResumenItem implements Comparable {

	/**
	 * Nombre del tem (nombre del archivo XML).
	 */
	private String nombre;
	/**
	 * Ttulo del tem.
	 */
	private String titulo;
	/**
	 * File al archivo XML del tem.
	 */
	private File archivo;

	/**
	 * Constructor de la clase que simplemente da a las variables de clase los valores 
	 * recibidos como parmetros.
	 * 
	 * @param 	nombre		string con el nombre del tem
	 * @param 	titulo		string con el ttulo del tem
	 * @param 	archivo		file representando al archivo XML del tem
	 */
	public ResumenItem(String nombre, String titulo, File archivo) {
		super();
		this.nombre = nombre;
		this.titulo = titulo;
		this.archivo = archivo;
	}

	/**
	 * Mtodo "get" que devuelve el <code>String</code> con el nombre del tem.
	 * 
	 * @return			string con el nombre del tem
	 */
	public String getNombre() {
		return nombre;
	}

	/**
	 * Mtodo "get" que devuelve el <code>String</code> con el ttulo del tem.
	 * 
	 * @return			string con el ttulo del tem
	 */
	public String getTitulo() {
		return titulo;
	}

	/**
	 * Mtodo "get" que devuelve el objeto <code>File</code> que representa al archivo XML del 
	 * tem.
	 * 
	 * @return			file con el archivo XML del tem
	 */
	public File getArchivo() {
		return archivo;
	}

	/**
	 * Mtodo definido por la interfaz <code>Comparable</code> para permitir la comparacin 
	 * entre los objetos que implementen la interfaz. Este mtodo utiliza el mtodo del mismo 
	 * nombre implementado por la clase <code>String</code> para comparar los objetos <code>
	 * ResumenItem</code> segn sus nombres. De este modo se permite ordenar de forma correcta 
	 * colecciones de objetos <code>ResumenItem</code> segn el nombre del tem. Para su 
	 * correcto funcionamiento, el mtodo comprueba primero que el objeto con el que se compara 
	 * sea tambin de tipo <code>ResumenItem</code>.
	 * 
	 * @param 	o					objeto con el que comparar este ResumenItem
	 * @return						entero resultado de la comparacin de los dos nombres de 
	 * 								los resmenes de los tems
	 * @throws	ClassCastException	si el objeto con el que comparar no es de este mismo tipo
	 */
	public int compareTo(Object o) {
		
		// Comprueba que el objeto "o" sea una instancia de esta misma clase
		if (!(o instanceof ResumenItem))
			throw new ClassCastException ("Se deben comparar dos objetos \"ResumenItem\"");
		else {
			ResumenItem ri = (ResumenItem) o;
			return nombre.compareTo(ri.nombre);
		}
	}
}