package utilidades.etiquetas.util;

import javax.servlet.jsp.tagext.SimpleTagSupport;
import javax.servlet.jsp.*;

import java.io.IOException;

/**
 * Etiqueta de pgina JSP para imprimir en pantalla la direccin de correo electrnico del 
 * administrador de la Aplicacin. La clase extiende a la clase <code>SimpleTagSupport</code> 
 * que representa a una Etiqueta Personalizada de JSP Simple. La etiqueta se encarga de 
 * devolver por la salida de la pgina JSP la direccin de correo electrnico de contacto del 
 * administrador, parmetro definido en el descriptor de despliegue de la Aplicacin, 
 * "web.xml". El proceso se realiza en el mtodo sobreescrito <code>doTag</code>, que se 
 * ejecuta al llamar a la etiqueta en una pgina JSP.
 * 
 * @author David Domnguez
 *
 */
public class AdminMailTag extends SimpleTagSupport {

	/**
	 * Imprime por pantalla la direccin de correo electrnico del administrador de la 
	 * Aplicacin. Recoge el parmetro definido en el descriptor de despliegue de la Aplicacin 
	 * ledo del contexto, y lo imprime por la salida estndar de la pgina JSP desde la que se 
	 * llame a la etiqueta. 
	 * 
	 * @throws 	IOException		si hay algn problema al intentar escribir en la JSP 
	 * @see 	javax.servlet.jsp.tagext.SimpleTag#doTag()
	 */
	public void doTag() throws IOException {
		// Contexto JSP para obtener el "Writer" para escribir en la pgina JSP
		JspContext jspC = getJspContext();
		JspWriter out = jspC.getOut();
		// Contexto de la pgina para acceder al contexto del Servlet y obtener el parmetro
		// requerido
		PageContext pc = (PageContext)jspC;
		String mail = pc.getServletContext().getInitParameter("admin email");
		// Se imprime en pantalla el parmetro recogido
		out.print(mail);
	}
}