package utilidades.etiquetas.util;

import java.io.IOException;
import java.net.URLDecoder;

import javax.servlet.jsp.JspContext;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.SimpleTagSupport;

/**
 * Etiqueta de pgina JSP para imprimir en pantalla la direccin de la pgina Web que caus el 
 * error. La clase extiende a la clase <code>SimpleTagSupport</code> que representa a una 
 * Etiqueta Personalizada de JSP Simple. La etiqueta se encarga de devolver por la salida de la 
 * pgina JSP, que debe ser una pgina de error, la URI que provoc el error. El proceso se 
 * realiza en el mtodo sobreescrito <code>doTag</code>, que se ejecuta al llamar a la etiqueta 
 * en una pgina JSP.
 * 
 * @author David Domnguez
 *
 */
public class RequestedURITag extends SimpleTagSupport {

	/**
	 * Imprime por pantalla la direccin de la Web que provoc el error. Recoge el parmetro 
	 * contenido en el mbito de request de la pgina JSP desde la que se llama  a la etiqueta, 
	 * que debe ser una pgina de error, indicando la uri que provoc ese error, y lo imprime 
	 * por la salida estndar de la pgina JSP desde la que se llame a la etiqueta. 
	 * 
	 * @throws 	IOException		si hay algn problema al intentar escribir en la JSP
	 * @see 	javax.servlet.jsp.tagext.SimpleTag#doTag()
	 */
	public void doTag() throws IOException{
		
		// Contexto de la pgina JSP para obtener el "Writer" para escribir en la pgina
		JspContext jspC = getJspContext();
		JspWriter out = jspC.getOut();
		// Se obtiene del mbito de request el valor de la uri que caus el error y se imprime 
		// en la pgina JSP
		String uri = URLDecoder.decode((String)jspC.
				getAttribute("javax.servlet.error.request_uri", PageContext.REQUEST_SCOPE), 
				"UTF-8");
		out.print(uri);
	}

}