package utilidades.etiquetas.util.logging;

import java.util.logging.ErrorManager;

/**
 * Manejador de los errores que se produzcan en el <code>Logger</code> de la Aplicacin Web. La 
 * clase se encarga de informar de que se ha producido algn error en el proceso del registro 
 * de errores. Sobreescribe el mtodo <code>error</code> que ser el llamado en caso de error 
 * al intentar registrar algo con el logger de la Aplicacin.
 * 
 * @author 	David Domnguez
 * @see		java.util.logging.ErrorManager
 */
public class ErrorManagerPersonalizado extends ErrorManager {

	/**
	 * Constructor de la clase que simplemente llama al constructor de la superclase.
	 */
	public ErrorManagerPersonalizado() {
		super();
	}

	
	/**
	 * Informa de que se ha producido un error al registrar algn evento de la Aplicacin Web. 
	 * El mtodo ser llamado cuando se produzca algn error al realizar algn registro. Este 
	 * mtodo mira qu tipo de error se ha producido y genera en consecuencia un mensaje a 
	 * medida, aadiendo tambin el mensaje que informa del error y el mensaje de la excepcin, 
	 * y sacndolo todo finalmente por la salida estndar de errores.
	 * 
	 * @param	msg		string con el mensaje informando del error ocurrido en el logger
	 * @param	ex		excepion con la excepcin que ha provocado el error del logger
	 * @param	code	entero con el cdigo de error que indica el tipo de error ocurrido
	 * @see java.util.logging.ErrorManager#error(java.lang.String, java.lang.Exception, int)
	 */
	public void error(String msg, Exception ex, int code) {
		// Mensaje de salida para informar del error producido
		String cadena = "";
		// Se crea un mensaje en funcin del tipo de error que se haya producido
		switch (code){
			case ErrorManager.CLOSE_FAILURE:
				cadena = "Error al cerrar el flujo de salida del manejador del logger de la " +
						"aplicacin QTI\n";
				break;
			case ErrorManager.FLUSH_FAILURE:
				cadena = "Error al realizar el \"flush\" sobre el flujo de salida del " +
						"manejador del logger de la aplicacin QTI\n";
				break;
			case ErrorManager.FORMAT_FAILURE:
				cadena = "Error al dar formato en el manejador del logger de la aplicacin " +
						"QTI\n";
				break;
			case ErrorManager.GENERIC_FAILURE:
				cadena = "Error genrico en el manejador del logger de la aplicacin QTI\n";
				break;
			case ErrorManager.OPEN_FAILURE:
				cadena = "Error al abrir el flujo de salidad del manejador del logger de la " +
						"aplicacin QTI\n";
				break;
			case ErrorManager.WRITE_FAILURE:
				cadena = "Error al escribir en el flujo de salida del manejador del logger " +
						"de la aplicacin QTI\n";
				break;
		}
		
		// Se aade al mensaje de salida el mensaje recibido y el mensaje de la excepcin 
		cadena += msg + ex.getMessage();
		// Se saca por la salida estndar de errores el mensaje creado
		System.err.println(cadena);
	}
}