package xml.test;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 * Clase que representa un objeto de control de la sesin al realizar el Test segn la norma 
 * QTI del IMS. Contiene slo una variable, que se corresponde con el atributo "allowReview" 
 * definido en la norma.
 * 
 * @author David Domnguez
 *
 */
public class ItemSessionControl {

	/**
	 * Permitir al candidato revisar la seccin tras responderla con las respuestas 
	 * que ha dado.
	 */
	boolean allowReview;
	
	/**
	 * Constructor que establece el valor del nico campo de la clase, <code>allowReview</code>.
	 * 
	 * @param 	allowReview		permitir o no revisar el examen tras responderlo
	 */
	public ItemSessionControl (boolean allowReview) {
		this.allowReview = allowReview;
	}
	
	/**
	 * Mtodo que escribe este objeto <code>ItemSessionControl</code> en el <code>
	 * XMLStreamWriter</code> recibido como parmetro, segn la norma QTI definida.
	 * 
	 * @param 	xsw						xmlStreamWriter con el que escribir el elemento 
	 * 									correspondiente a esta clase en el archivo XML
	 * @throws 	XMLStreamException		si hay algn problema al escribir con el <code>
	 * 									XMLStremaWriter</code> los datos XML en el archivo
	 * @see		javax.xml.stream.XMLStreamWriter
	 */
	public void creaItemSessionControl(XMLStreamWriter xsw) throws XMLStreamException{

		xsw.writeEmptyElement("itemSessionControl");	// Escribe el elemento vaco
		xsw.writeAttribute("allowReview", Boolean.toString(allowReview)); // Escribe el Atributo
	}
}