package xml.test;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 * Clase que representa el criterio de ordenacin de los tems de una seccin de un Test segn 
 * la norma QTI del IMS. El nico criterio definido por la norma es permitir mezclar los tems
 * aleatoriamente o dejarlos en la posicin definida en el Test. Contiene un atributo que se 
 * corresponde con el definido en la norma.
 * 
 * @author David Domnguez
 *
 */
public class Ordering {

	/**
	 * Variable indicando si se deben mezclar aleatoriamente los tems o no.
	 */
	private boolean shuffle;
	
	/**
	 * Constructor que simplemente inicializa la variable de la clase boolean.
	 * 
	 * @param 	shuffle		boolean indicando si se mezclan aleatoriamente los tems
	 */
	public Ordering (boolean shuffle) {
		this.shuffle = shuffle;
	}

	/**
	 * Mtodo que escribe este objeto <code>Ordering</code> en el <code>XMLStreamWriter</code> 
	 * recibido como parmetro, segn la norma QTI definida.
	 * 
	 * @param 	xsw						xmlStreamWriter con el que escribir el elemento 
	 * 									correspondiente a esta clase en el archivo XML
	 * @throws 	XMLStreamException		si hay algn problema al escribir con el <code>
	 * 									XMLStremaWriter</code> los datos XML en el archivo
	 * @see		javax.xml.stream.XMLStreamWriter
	 */
	public void creaOrdering(XMLStreamWriter xsw) throws XMLStreamException{
		
		xsw.writeEmptyElement("ordering");		// Escribe el elemento
		xsw.writeAttribute("shuffle", Boolean.toString(shuffle));	// Atributo del elemento
	}
}