package xml.test;

import utilidades.Identificador;

/**
 * Clase que representa a la clase abstracta del mismo nombre segn la norma QTI del IMS. Es la 
 * superclase de otras clases que componen un Test. Contiene los atributos que se corresponden 
 * con los definidos en la norma.
 * 
 * @author David Domnguez
 *
 */
public abstract class SectionPart {

	/**
	 * Identificador del elemento.
	 */
	protected Identificador identificador;
	/**
	 * boolean indicando si la subclase est requerida obligatoriamente en caso de que se use 
	 * algn criterio de seleccin.
	 */
	protected boolean requerido;
	/**
	 * boolean indicando si la subclase est fija en caso de que se use algn criterio de 
	 * ordenacin.
	 */
	protected boolean fijo;
		
	/**
	 * Constructor de la clase que simplemente le da valores iniciales a las variables de clase.
	 * 
	 * @param 	identificador	identificador que contiene un identificador vlido para la clase
	 * @param 	requerido		boolean indicando que este elemento es requerido
	 * @param 	fijo			boolean indicando que este elemento est en posicin fija
	 */
	public SectionPart(Identificador identificador, boolean requerido, boolean fijo) {
		this.identificador = identificador;
		this.requerido = requerido;
		this.fijo = fijo;
	}

	/**
	 * Mtodo "get" para obtener el identificador de la clase.
	 * 
	 * @return		identificador que contiene el identificador vlido de la clase
	 */
	public Identificador getIdentificador() {
		return identificador;
	}
}