package xml.test;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 * Clase que representa el criterio de seleccin de los tems de una seccin de un Test segn 
 * la norma QTI del IMS. El nico criterio definido por la norma es seleccionar un subconjunto 
 * del conjunto total de tems de la seccin. Contiene unos atributos que se correspondes con 
 * los definidos en la norma.
 * 
 * @author David Domnguez
 *
 */
public class Selection {
	
	/**
	 * Nmero de tems a seleccionar del nmero total de tems.
	 */
	private int seleccion;
	/**
	 * Si se pueden repetir los tems escogindolos ms de una vez o no.
	 */
	private boolean conReemplazamiento;

	/**
	 * Constructor de la clase que simplemente le da valores iniciales a las variables de la 
	 * clase.
	 * 
	 * @param 	seleccion			entero con el nmero de tems a seleccionar del conjunto
	 * @param 	conReemplazamiento	boolean indicando si se puede escoger varias veces el mismo 
	 * 								tem para formar el Test final
	 */
	public Selection (int seleccion, boolean conReemplazamiento) {
		this.seleccion = seleccion;
		this.conReemplazamiento = conReemplazamiento;
	}
	
	/**
	 * Mtodo "get" que devuelve el nmero de tems a seleccionar del conjunto de los tems de 
	 * la seccin.
	 * 
	 * @return		entero indicando el nmero de tems a seleccionar
	 */
	public int getSeleccion() {
		return seleccion;
	}

	/**
	 * Mtodo que escribe este objeto <code>Selection</code> en el <code>XMLStreamWriter</code> 
	 * recibido como parmetro, segn la norma QTI definida.
	 * 
	 * @param 	xsw						xmlStreamWriter con el que escribir el elemento 
	 * 									correspondiente a esta clase en el archivo XML
	 * @throws 	XMLStreamException		si hay algn problema al escribir con el <code>
	 * 									XMLStremaWriter</code> los datos XML en el archivo
	 * @see		javax.xml.stream.XMLStreamWriter
	 */
	public void creaSelection (XMLStreamWriter xsw) throws XMLStreamException{
		xsw.writeEmptyElement("selection ");		// Escribe el elemento
		xsw.writeAttribute("select", Integer.toString(seleccion));	// Atributo seleccin
		// Atributo con reemplazamiento
		xsw.writeAttribute("withReplacement", Boolean.toString(conReemplazamiento));
	}
}