package xml.test;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import utilidades.Identificador;

/**
 * Clase que representa un Peso de un tem segn la norma QTI del IMS. Se utiliza para aplicar 
 * pesos distintos a los distintos tems que componen el Test a la hora de evaluarlos. Contiene 
 * unos atributos que se corresponden con los definidos en la norma.
 * 
 * @author David Domnguez
 *
 */
public class Weight {

	/**
	 * Identificador vlido del peso.
	 */
	private Identificador identificador;
	/**
	 * Valor del peso que tiene el tem dentro del Test.
	 */
	private double peso;
	
	/**
	 * Constructor que simplemente le da valores iniciales a las variables de la clase.
	 * 
	 * @param 	identificador	identificador con el identificador vlido de la clase
	 * @param 	peso			double con el valor del peso del tem en el Test
	 */
	public Weight (Identificador identificador, double peso) {
		this.identificador = identificador;
		this.peso = peso;
	}
	
	/**
	 * Mtodo que escribe este objeto <code>Weight</code> en el <code>XMLStreamWriter</code> 
	 * recibido como parmetro, segn la norma QTI definida.
	 * 
	 * @param 	xsw						xmlStreamWriter con el que escribir el elemento 
	 * 									correspondiente a esta clase en el archivo XML
	 * @throws 	XMLStreamException		si hay algn problema al escribir con el <code>
	 * 									XMLStremaWriter</code> los datos XML en el archivo
	 * @see		javax.xml.stream.XMLStreamWriter
	 */
	public void creaWeight (XMLStreamWriter xsw) throws XMLStreamException{
		xsw.writeStartElement("weight");				// Escribe el elemento
		xsw.writeAttribute("identifier", identificador.getIdentificador());	// Identificador
		xsw.writeCharacters(Double.toString(peso));		// Valor del peso 
		xsw.writeEndElement();							// Cierra "weight"
	}
}